@@ -39,7 +39,9 @@ static pthread_mutex_t lock = PTHREAD_MUTEX_INITIALIZER;
 #define BUFREAD64(buf,var) memcpy(&var, buf, 8); if (diff_endian) swap_64(&var);
 #endif
 
-/* TODO: the following 3 global variables make PnetCDF not thread safe */
+/* Note accessing the following 3 global variables must be protected by a
+ * mutex, otherwise it will not be thread safe.
+ */
 
 /* static variables are initialized to NULLs */
 static PNC *pnc_filelist[NC_MAX_NFILES];
@@ -50,14 +52,23 @@ static int  pnc_numfiles;
  */
 static int ncmpi_default_create_format = NC_FORMAT_CLASSIC;
 
-#define NCMPII_HANDLE_ERROR(func) \
-    if (mpireturn != MPI_SUCCESS) { \
-        int errorStringLen; \
-        char errorString[MPI_MAX_ERROR_STRING]; \
+#define NCMPII_HANDLE_ERROR(func)                                  \
+    if (mpireturn != MPI_SUCCESS) {                                \
+        int errorStringLen;                                        \
+        char errorString[MPI_MAX_ERROR_STRING];                    \
         MPI_Error_string(mpireturn, errorString, &errorStringLen); \
-        printf("%s error at line %d file %s (%s)\n", func, __LINE__, __FILE__, errorString); \
+        printf("Error: file %s line %d calling func %s: (%s)\n",   \
+               __FILE__, __LINE__, func, errorString);             \
     }
 
+#define CHECK_ERRNO(err, func) {                                   \
+    if (err != 0) {                                                \
+        printf("Error: file %s line %d calling func %s: (%s)\n",   \
+               __FILE__, __LINE__, func, strerror(err));           \
+        goto err_out;                                              \
+    }                                                              \
+}
+
 /*----< new_id_PNCList() >---------------------------------------------------*/
 /* Return a new ID (array index) from the PNC list, pnc_filelist[] that is
  * not used. Note the used elements in pnc_filelist[] may not be contiguous.
@@ -67,10 +78,11 @@ static int ncmpi_default_create_format = NC_FORMAT_CLASSIC;
 static int
 new_id_PNCList(int *new_id, PNC *pncp)
 {
-    int i, err;
+    int i, err=NC_NOERR, perr=0;
 
 #ifdef ENABLE_THREAD_SAFE
-    pthread_mutex_lock(&lock);
+    perr = pthread_mutex_lock(&lock);
+    CHECK_ERRNO(perr, "pthread_mutex_lock")
 #endif
     *new_id = -1;
     if (pnc_numfiles == NC_MAX_NFILES) { /* Too many files open */
@@ -88,18 +100,23 @@ new_id_PNCList(int *new_id, PNC *pncp)
         }
     }
 #ifdef ENABLE_THREAD_SAFE
-    pthread_mutex_unlock(&lock);
-#endif
+    perr = pthread_mutex_unlock(&lock);
+    CHECK_ERRNO(perr, "pthread_mutex_unlock")
 
-    return err;
+err_out:
+#endif
+    return (err != NC_NOERR) ? err : perr;
 }
 
 /*----< del_from_PNCList() >-------------------------------------------------*/
-static void
+static int
 del_from_PNCList(int ncid)
 {
+    int perr=0;
+
 #ifdef ENABLE_THREAD_SAFE
-    pthread_mutex_lock(&lock);
+    perr = pthread_mutex_lock(&lock);
+    CHECK_ERRNO(perr, "pthread_mutex_lock")
 #endif
 
     /* validity of ncid should have been checked already */
@@ -107,20 +124,25 @@ del_from_PNCList(int ncid)
     pnc_numfiles--;
 
 #ifdef ENABLE_THREAD_SAFE
-    pthread_mutex_unlock(&lock);
+    perr = pthread_mutex_unlock(&lock);
+    CHECK_ERRNO(perr, "pthread_mutex_unlock")
+
+err_out:
 #endif
+    return perr;
 }
 
 /*----< PNC_check_id() >-----------------------------------------------------*/
 int
 PNC_check_id(int ncid, PNC **pncp)
 {
-    int err=NC_NOERR;
+    int err=NC_NOERR, perr=0;
 
     assert(pncp != NULL);
 
 #ifdef ENABLE_THREAD_SAFE
-    pthread_mutex_lock(&lock);
+    perr = pthread_mutex_lock(&lock);
+    CHECK_ERRNO(perr, "pthread_mutex_lock")
 #endif
 
     if (pnc_numfiles == 0 || ncid < 0 || ncid >= NC_MAX_NFILES)
@@ -129,10 +151,12 @@ PNC_check_id(int ncid, PNC **pncp)
         *pncp = pnc_filelist[ncid];
 
 #ifdef ENABLE_THREAD_SAFE
-    pthread_mutex_unlock(&lock);
-#endif
+    perr = pthread_mutex_unlock(&lock);
+    CHECK_ERRNO(perr, "pthread_mutex_unlock")
 
-    return err;
+err_out:
+#endif
+    return (err != NC_NOERR) ? err : perr;
 }
 
 /*----< construct_info() >---------------------------------------------------*/
@@ -762,6 +786,7 @@ ncmpi_open(MPI_Comm    comm,
     pncp->ndims      = 0;
     pncp->unlimdimid = -1;
     pncp->nvars      = 0;
+    pncp->nrec_vars  = 0;
     pncp->vars       = NULL;
     pncp->flag       = 0;
     pncp->ncp        = ncp;
@@ -822,6 +847,7 @@ ncmpi_open(MPI_Comm    comm,
                 if (err != NC_NOERR) break; /* loop i */
             }
         }
+        if (pncp->vars[i].recdim >= 0) pncp->nrec_vars++;
     }
     if (err != NC_NOERR) { /* error happens in loop i */
         assert(i < pncp->nvars);
@@ -1185,13 +1211,12 @@ ncmpi_inq_file_format(const char *filename,
 
     *formatp = NC_FORMAT_UNKNOWN;
 
-    /* remove the file system type prefix name if there is any.
-     * For example, when filename = "lustre:/home/foo/testfile.nc", remove
-     * "lustre:" to make path = "/home/foo/testfile.nc" in open() below
+    /* remove the file system type prefix name if there is any.  For example,
+     * when filename = "lustre:/home/foo/testfile.nc", remove "lustre:" to make
+     * path pointing to "/home/foo/testfile.nc", so it can be used in POSIX
+     * open() below
      */
-    path = strchr(filename, ':');
-    if (path == NULL) path = filename; /* no prefix */
-    else              path++;
+    path = ncmpii_remove_file_system_type_prefix(filename);
 
     /* must include config.h on 32-bit machines, as AC_SYS_LARGEFILE is called
      * at the configure time and it defines _FILE_OFFSET_BITS to 64 if large
@@ -1740,10 +1765,11 @@ ncmpi_sync_numrecs(int ncid)
 int
 ncmpi_set_default_format(int format, int *old_formatp)
 {
-    int err;
+    int err=NC_NOERR, perr=0;
 
 #ifdef ENABLE_THREAD_SAFE
-    pthread_mutex_lock(&lock);
+    perr = pthread_mutex_lock(&lock);
+    CHECK_ERRNO(perr, "pthread_mutex_lock")
 #endif
 
     /* Return existing format if desired. */
@@ -1764,32 +1790,38 @@ ncmpi_set_default_format(int format, int *old_formatp)
     }
 
 #ifdef ENABLE_THREAD_SAFE
-    pthread_mutex_unlock(&lock);
-#endif
+    perr = pthread_mutex_unlock(&lock);
+    CHECK_ERRNO(perr, "pthread_mutex_unlock")
 
-    return err;
+err_out:
+#endif
+    return (err != NC_NOERR) ? err : perr;
 }
 
 /*----< ncmpi_inq_default_format() >-----------------------------------------*/
 /* returns a value suitable for a create flag.  Will return one or more of the
- * following values OR-ed together:
- * NC_64BIT_OFFSET, NC_CLOBBER, NC_LOCK, NC_SHARE */
+ * following values OR-ed together: NC_64BIT_OFFSET, NC_CLOBBER */
 int
 ncmpi_inq_default_format(int *formatp)
 {
+    int perr=0;
+
     if (formatp == NULL) DEBUG_RETURN_ERROR(NC_EINVAL)
 
 #ifdef ENABLE_THREAD_SAFE
-    pthread_mutex_lock(&lock);
+    perr = pthread_mutex_lock(&lock);
+    CHECK_ERRNO(perr, "pthread_mutex_lock")
 #endif
 
     *formatp = ncmpi_default_create_format;
 
 #ifdef ENABLE_THREAD_SAFE
-    pthread_mutex_unlock(&lock);
-#endif
+    perr = pthread_mutex_unlock(&lock);
+    CHECK_ERRNO(perr, "pthread_mutex_unlock")
 
-    return NC_NOERR;
+err_out:
+#endif
+    return perr;
 }
 
 /*----< ncmpi_inq_files_opened() >-------------------------------------------*/
@@ -1798,12 +1830,13 @@ int
 ncmpi_inq_files_opened(int *num,    /* cannot be NULL */
                        int *ncids)  /* can be NULL */
 {
-    int i;
+    int i, perr=0;
 
     if (num == NULL) DEBUG_RETURN_ERROR(NC_EINVAL)
 
 #ifdef ENABLE_THREAD_SAFE
-    pthread_mutex_lock(&lock);
+    perr = pthread_mutex_lock(&lock);
+    CHECK_ERRNO(perr, "pthread_mutex_lock")
 #endif
 
     *num = pnc_numfiles;
@@ -1818,10 +1851,12 @@ ncmpi_inq_files_opened(int *num,    /* cannot be NULL */
         }
     }
 #ifdef ENABLE_THREAD_SAFE
-    pthread_mutex_unlock(&lock);
-#endif
+    perr = pthread_mutex_unlock(&lock);
+    CHECK_ERRNO(perr, "pthread_mutex_unlock")
 
-    return NC_NOERR;
+err_out:
+#endif
+    return perr;
 }
 
 /*----< ncmpi_inq_nreqs() >--------------------------------------------------*/
