@@ -71,22 +71,21 @@ dtype_filter(MPI_Datatype type)
 
     /* 4-byte integer types */
   {
-    MPI_Datatype int_4byte;
 #if (SIZEOF_LONG == 4)
     MPI_Datatype uint_4byte;
 #endif
 #if (SIZEOF_INT == 4)
-     int_4byte = MPI_INT;
+     MPI_Datatype int_4byte = MPI_INT;
 #if (SIZEOF_LONG == 4)
     uint_4byte = MPI_UNSIGNED;
 #endif
 #elif (SIZEOF_SHORT == 4)
-     int_4byte = MPI_SHORT;
+     MPI_Datatype int_4byte = MPI_SHORT;
 #if (SIZEOF_LONG == 4)
     uint_4byte = MPI_UNSIGNED_SHORT;
 #endif
 #else
-     int_4byte = MPI_DATATYPE_NULL; /* no 4-byte type? */
+     MPI_Datatype int_4byte = MPI_DATATYPE_NULL; /* no 4-byte type? */
 #if (SIZEOF_LONG == 4)
     uint_4byte = MPI_DATATYPE_NULL;
 #endif
@@ -481,9 +480,9 @@ int ncmpii_dtype_decode(MPI_Datatype  dtype,
 /*----< ncmpii_buftype_decode() >--------------------------------------------*/
 /* Obtain the following metadata about buftype:
  * etype:    element data type (MPI primitive type) in buftype
- * bufcount: If it is -1, then this is called from a high-level API and in
- *           this case buftype will be an MPI primitive data type.
- *           If bufcount is not -1, then this is called from a flexible API.
+ * bufcount: If it is NC_COUNT_IGNORE, then this is called from a high-level
+ *           API and in this case buftype must be an MPI predefined data type.
+ *           Otherwise, this is called from a flexible API.
  * nelems:   number of etypes in user buffer
  * xnbytes:  number of bytes (in external data representation) to read/write
  *           from/to the file
@@ -516,14 +515,7 @@ ncmpii_buftype_decode(int               ndims,
     for (i=0; i<ndims; i++)
         fnelems *= count[i];
 
-    if (bufcount == -1) { /* the subroutine is called from a high-level API */
-        *nelems   = fnelems;
-        *etype    = buftype; /* buftype is an MPI primitive data type */
-        MPI_Type_size(buftype, esize);
-        *xnbytes  = *nelems * xsz;
-        *isContig = 1;
-    }
-    else if (buftype == MPI_DATATYPE_NULL) {
+    if (buftype == MPI_DATATYPE_NULL) {
         /* This is called from a flexible API and buftype is set by user to
          * MPI_DATATYPE_NULL. In this case, bufcount is ignored and set by
          * this subroutine to a number match count[], and etype to match the
@@ -535,10 +527,20 @@ ncmpii_buftype_decode(int               ndims,
         *xnbytes  = *nelems * xsz;
         *isContig = 1;
     }
+    else if (bufcount == NC_COUNT_IGNORE) {
+        /* This is called from a high-level API, buftype must be a predefined
+         * MPI datatype. This requirement has already been checked in
+         * src/dispatchers/var_getput.m4
+         */
+        MPI_Type_size(buftype, esize);
+        *etype    = buftype;
+        *nelems   = fnelems;
+        *xnbytes  = *nelems * xsz;
+        *isContig = 1;
+    }
     else { /* This is called from a flexible API */
-        int isderived;
         /* check some metadata of the MPI derived datatype */
-        err = ncmpii_dtype_decode(buftype, etype, esize, nelems, &isderived,
+        err = ncmpii_dtype_decode(buftype, etype, esize, nelems, NULL,
                                   isContig);
         if (err != NC_NOERR) return err;
 
