@@ -12,7 +12,6 @@
 #ifdef HAVE_STDLIB_H
 #include <stdlib.h>
 #endif
-#include <limits.h>  /* INT_MAX */
 #include <assert.h>
 
 #include <mpi.h>
@@ -106,140 +105,6 @@ is_request_contiguous(int               isRecVar,
     return 1;
 }
 
-#ifndef HAVE_MPI_TYPE_CREATE_SUBARRAY
-/*----< type_create_subarray() >---------------------------------------------*/
-/* this is to be used when MPI_Type_create_subarray() is not available,
- * typically for MPI-1 implementation only */
-static int
-type_create_subarray(int           ndims,
-                     const int    *array_of_sizes,    /* [ndims] */
-                     const int    *array_of_subsizes, /* [ndims] */
-                     const int    *array_of_starts,   /* [ndims] */
-                     int           order,
-                     MPI_Datatype  oldtype,
-                     MPI_Datatype *newtype)
-{
-    int i, err, blklens[3] = {1, 1, 1};
-    MPI_Datatype type1, type2;
-    MPI_Aint extent, size, array_size, stride, disps[3];
-
-    if (ndims == 0) DEBUG_RETURN_ERROR(NC_EDIMMETA)
-
-#ifdef HAVE_MPI_TYPE_GET_EXTENT
-    MPI_Aint lb;
-    MPI_Type_get_extent(oldtype, &lb, &extent);
-#else
-    MPI_Type_extent(oldtype, &extent);
-#endif
-    array_size = extent;
-    for (i=0; i<ndims; i++) array_size *= array_of_sizes[i];
-
-    if (ndims == 1) {
-        /* blklens argument in MPI_Type_create_hindexed() is of type int */
-        blklens[1] = array_of_subsizes[0];
-        disps[1] = extent * array_of_starts[0];
-
-#if defined (HAVE_MPI_TYPE_CREATE_HINDEXED) && defined(HAVE_MPI_TYPE_CREATE_RESIZED)
-        /* take advantage of disps argument is of type MPI_Aint */
-        err = MPI_Type_create_hindexed(1, &blklens[1], &disps[1], oldtype, &type1);
-        if (err != MPI_SUCCESS)
-            return ncmpii_error_mpi2nc(err, "MPI_Type_create_hindexed");
-        MPI_Type_commit(&type1);
-
-        /* add holes in the beginning and tail of type1 */
-        err = MPI_Type_create_resized(type1, 0, array_size, newtype);
-        if (err != MPI_SUCCESS)
-            return ncmpii_error_mpi2nc(err, "MPI_Type_create_resized");
-        MPI_Type_free(&type1);
-#else
-        /* add holes in the beginning and tail of oldtype */
-        MPI_Datatype types[3];
-        types[0] = MPI_LB; types[1] = oldtype; types[2] = MPI_UB;
-        disps[0] = 0;                          disps[2] = array_size;
-        err = MPI_Type_struct(3, blklens, disps, types, newtype);
-        if (err != MPI_SUCCESS)
-            return ncmpii_error_mpi2nc(err, "MPI_Type_struct");
-#endif
-        return NC_NOERR;
-    }
-    /* now, ndims > 1 */
-
-    /* first create a datatype for the least 2 significant dimensions */
-
-    /* blklens argument in MPI_Type_create_hvector() is of type int */
-    blklens[0] = array_of_subsizes[ndims-1];
-    stride = array_of_sizes[ndims-1] * extent;
-#ifdef HAVE_MPI_TYPE_CREATE_HVECTOR
-    err = MPI_Type_create_hvector(array_of_subsizes[ndims-2], blklens[0],
-                                  stride, oldtype, &type1);
-    if (err != MPI_SUCCESS)
-        return ncmpii_error_mpi2nc(err, "MPI_Type_create_hvector");
-#else
-    err = MPI_Type_hvector(array_of_subsizes[ndims-2], blklens[0],
-                           stride, oldtype, &type1);
-    if (err != MPI_SUCCESS)
-        return ncmpii_error_mpi2nc(err, "MPI_Type_hvector");
-#endif
-    MPI_Type_commit(&type1);
-
-    /* now iterate through the rest dimensions */
-    for (i=ndims-3; i>=0; i--) {
-        stride *= array_of_sizes[i+1];
-#ifdef HAVE_MPI_TYPE_CREATE_HVECTOR
-        err = MPI_Type_create_hvector(array_of_subsizes[i], 1, stride, type1, &type2);
-        if (err != MPI_SUCCESS)
-            return ncmpii_error_mpi2nc(err, "MPI_Type_create_hvector");
-#else
-        err = MPI_Type_hvector(array_of_subsizes[i], 1, stride, type1, &type2);
-        if (err != MPI_SUCCESS)
-            return ncmpii_error_mpi2nc(err, "MPI_Type_hvector");
-#endif
-        MPI_Type_commit(&type2);
-        MPI_Type_free(&type1);
-        type1 = type2;
-    }
-
-    /* disps[1] is the first byte displacement of the subarray */
-    disps[1] = array_of_starts[ndims-1] * extent;
-    size = 1;
-    for (i=ndims-2; i>=0; i--) {
-        size *= array_of_sizes[i+1];
-        disps[1] += size * array_of_starts[i];
-    }
-
-    /* disps[2] is the size of the global array */
-    disps[2] = array_size;
-
-    /* disps[0] is the beginning of the global array */
-    disps[0] = 0;
-
-    /* make filetype the same as calling MPI_Type_create_subarray() */
-    blklens[0] = 1;
-#if defined (HAVE_MPI_TYPE_CREATE_HINDEXED) && defined(HAVE_MPI_TYPE_CREATE_RESIZED)
-    /* adjust LB and UB without using MPI_LB or MPI_UB */
-    err = MPI_Type_create_hindexed(1, blklens, &disps[1], type1, &type2);
-    if (err != MPI_SUCCESS)
-        return ncmpii_error_mpi2nc(err, "MPI_Type_create_hindexed");
-    MPI_Type_commit(&type2);
-    err = MPI_Type_create_resized(type2, disps[0], disps[2], newtype);
-    if (err != MPI_SUCCESS)
-        return ncmpii_error_mpi2nc(err, "MPI_Type_create_resized");
-    MPI_Type_free(&type2);
-#else
-    MPI_Datatype types[3];
-    types[0] = MPI_LB;
-    types[1] = type1;
-    types[2] = MPI_UB;
-    err = MPI_Type_struct(3, blklens, disps, types, newtype);
-    if (err != MPI_SUCCESS)
-        return ncmpii_error_mpi2nc(err, "MPI_Type_struct");
-#endif
-    MPI_Type_free(&type1);
-
-    return NC_NOERR;
-}
-#endif
-
 /*----< type_create_subarray64() >-------------------------------------------*/
 /* This subroutine is to achieve the same result as MPI_Type_create_subarray()
  * but it takes arguments in type of MPI_Offset, instead of int. It also
@@ -254,32 +119,49 @@ type_create_subarray64(int               ndims,
                        MPI_Datatype      oldtype,
                        MPI_Datatype     *newtype)
 {
-    int i, err=NC_NOERR, mpireturn, tag, blklens[3] = {1, 1, 1};
-    MPI_Datatype type1, type2;
-    MPI_Aint extent, size, array_size, stride, disps[3];
-#ifdef HAVE_MPI_TYPE_GET_EXTENT
-    MPI_Aint lb;
-#endif
+    int i, err=NC_NOERR, mpireturn;
 
     if (ndims == 0) DEBUG_RETURN_ERROR(NC_EDIMMETA)
 
-    /* check if any of the dimensions is larger than 2^31-1 */
-    tag = 0;
+#ifdef HAVE_MPI_LARGE_COUNT
+    MPI_Count *sizes, *subsizes, *starts;
+
+    sizes = (MPI_Count*) NCI_Malloc((size_t)ndims * 3 * sizeof(MPI_Count));
+    subsizes = sizes    + ndims;
+    starts   = subsizes + ndims;
     for (i=0; i<ndims; i++) {
-        if (array_of_sizes[i] > INT_MAX || array_of_starts[i] > INT_MAX) {
-            tag = 1;
+        sizes[i]    = (MPI_Count)array_of_sizes[i];
+        subsizes[i] = (MPI_Count)array_of_subsizes[i];
+        starts[i]   = (MPI_Count)array_of_starts[i];
+    }
+    mpireturn = MPI_Type_create_subarray_c(ndims, sizes, subsizes, starts,
+                                           order, oldtype, newtype);
+    if (mpireturn != MPI_SUCCESS)
+        err = ncmpii_error_mpi2nc(mpireturn, "MPI_Type_create_subarray_c");
+
+    NCI_Free(sizes);
+    return err;
+
+#else
+    int big_int, blklens[3] = {1, 1, 1};
+    MPI_Datatype type1, type2;
+    MPI_Aint extent, size, array_size, stride, disps[3];
+    MPI_Aint lb;
+
+    /* check if any of the dimensions is larger than 2^31-1 */
+    big_int = 0;
+    for (i=0; i<ndims; i++) {
+        if (array_of_sizes[i] > NC_MAX_INT || array_of_starts[i] > NC_MAX_INT) {
+            big_int = 1;
             break;
         }
     }
 
-    if (tag == 0) {
-        int gsizes[24], *sizes, *subsizes, *starts;
+    if (big_int == 0) {
+        int *sizes, *subsizes, *starts;
         /* none of dimensions > 2^31-1, we can safely use
          * MPI_Type_create_subarray */
-        if (ndims <= 8) /* avoid malloc */
-            sizes = gsizes;
-        else
-            sizes = (int*) NCI_Malloc((size_t)ndims * 3 * SIZEOF_INT);
+        sizes = (int*) NCI_Malloc((size_t)ndims * 3 * SIZEOF_INT);
         subsizes = sizes    + ndims;
         starts   = subsizes + ndims;
         for (i=0; i<ndims; i++) {
@@ -287,19 +169,17 @@ type_create_subarray64(int               ndims,
             subsizes[i] = (int)array_of_subsizes[i];
             starts[i]   = (int)array_of_starts[i];
         }
-#ifdef HAVE_MPI_TYPE_CREATE_SUBARRAY
         mpireturn = MPI_Type_create_subarray(ndims, sizes, subsizes, starts,
                                              order, oldtype, newtype);
         if (mpireturn != MPI_SUCCESS)
             err = ncmpii_error_mpi2nc(mpireturn, "MPI_Type_create_subarray");
-#else
-        err = type_create_subarray(ndims, sizes, subsizes, starts,
-                                   order, oldtype, newtype);
-#endif
-        if (ndims > 8) NCI_Free(sizes);
+
+        NCI_Free(sizes);
         return err;
     }
 
+    /* now big_int = 1 */
+
     /* at least one dimension is of size > 2^31-1 and we cannot use
      * MPI_Type_create_subarray() to create the newtype,
      * as its arguments array_of_sizes[] and array_of_starts[] are of
@@ -316,22 +196,17 @@ type_create_subarray64(int               ndims,
     DEBUG_RETURN_ERROR(NC_EAINT_TOO_SMALL)
 #endif
 
-#ifdef HAVE_MPI_TYPE_GET_EXTENT
     MPI_Type_get_extent(oldtype, &lb, &extent);
-#else
-    MPI_Type_extent(oldtype, &extent);
-#endif
     array_size = extent;
     for (i=0; i<ndims; i++) array_size *= array_of_sizes[i];
 
     if (ndims == 1) {
         /* blklens argument in MPI_Type_create_hindexed() is of type int */
-        blklens[1] = (int)array_of_subsizes[0];
-        if (array_of_subsizes[0] != blklens[1]) /* check int overflow */
+        if (array_of_subsizes[0] > NC_MAX_INT) /* check int overflow */
             DEBUG_RETURN_ERROR(NC_EINTOVERFLOW)
+        blklens[1] = (int)array_of_subsizes[0];
         disps[1] = extent * array_of_starts[0];
 
-#if defined (HAVE_MPI_TYPE_CREATE_HINDEXED) && defined(HAVE_MPI_TYPE_CREATE_RESIZED)
         /* take advantage of disps argument is of type MPI_Aint */
         err = MPI_Type_create_hindexed(1, &blklens[1], &disps[1], oldtype, &type1);
         if (err != MPI_SUCCESS)
@@ -343,15 +218,7 @@ type_create_subarray64(int               ndims,
         if (err != MPI_SUCCESS)
             return ncmpii_error_mpi2nc(err, "MPI_Type_create_resized");
         MPI_Type_free(&type1);
-#else
-        /* add holes in the beginning and tail of oldtype */
-        MPI_Datatype types[3];
-        types[0] = MPI_LB; types[1] = oldtype; types[2] = MPI_UB;
-        disps[0] = 0;                          disps[2] = array_size;
-        err = MPI_Type_struct(3, blklens, disps, types, newtype);
-        if (err != MPI_SUCCESS)
-            return ncmpii_error_mpi2nc(err, "MPI_Type_struct");
-#endif
+
         return NC_NOERR;
     }
     /* now, ndims > 1 */
@@ -361,38 +228,33 @@ type_create_subarray64(int               ndims,
     /* count and blocklength arguments in MPI_Type_create_hvector() are of
      * type int. We need to check for integer overflow */
     int count, blocklength;
+
+    if (array_of_subsizes[ndims-2] > NC_MAX_INT ||
+        array_of_subsizes[ndims-1] > NC_MAX_INT) /* check int overflow */
+        DEBUG_RETURN_ERROR(NC_EINTOVERFLOW)
+
     count = (int)array_of_subsizes[ndims-2];
     blocklength = (int)array_of_subsizes[ndims-1];
-    if (array_of_subsizes[ndims-2] != count ||
-        array_of_subsizes[ndims-1] != blocklength) /* check int overflow */
-        DEBUG_RETURN_ERROR(NC_EINTOVERFLOW)
     stride = array_of_sizes[ndims-1] * extent;
-#ifdef HAVE_MPI_TYPE_CREATE_HVECTOR
+
     err = MPI_Type_create_hvector(count, blocklength, stride, oldtype, &type1);
     if (err != MPI_SUCCESS)
         return ncmpii_error_mpi2nc(err, "MPI_Type_create_hvector");
-#else
-    err = MPI_Type_hvector(count, blocklength, stride, oldtype, &type1);
-    if (err != MPI_SUCCESS)
-        return ncmpii_error_mpi2nc(err, "MPI_Type_hvector");
-#endif
+
     MPI_Type_commit(&type1);
 
     /* now iterate through the rest dimensions */
     for (i=ndims-3; i>=0; i--) {
-        count = (int)array_of_subsizes[i];
-        if (array_of_subsizes[i] != count) /* check int overflow */
+        if (array_of_subsizes[i] > NC_MAX_INT) /* check int overflow */
             DEBUG_RETURN_ERROR(NC_EINTOVERFLOW)
+
+        count = (int)array_of_subsizes[i];
         stride *= array_of_sizes[i+1];
-#ifdef HAVE_MPI_TYPE_CREATE_HVECTOR
+
         err = MPI_Type_create_hvector(count, 1, stride, type1, &type2);
         if (err != MPI_SUCCESS)
             return ncmpii_error_mpi2nc(err, "MPI_Type_create_hvector");
-#else
-        err = MPI_Type_hvector(count, 1, stride, type1, &type2);
-        if (err != MPI_SUCCESS)
-            return ncmpii_error_mpi2nc(err, "MPI_Type_hvector");
-#endif
+
         MPI_Type_commit(&type2);
         MPI_Type_free(&type1);
         type1 = type2;
@@ -413,7 +275,6 @@ type_create_subarray64(int               ndims,
     disps[2] = array_size;
 
     /* make filetype the same as calling MPI_Type_create_subarray() */
-#if defined(HAVE_MPI_TYPE_CREATE_HINDEXED) && defined(HAVE_MPI_TYPE_CREATE_RESIZED)
     /* adjust LB and UB without using MPI_LB or MPI_UB */
     err = MPI_Type_create_hindexed(1, blklens, &disps[1], type1, &type2);
     if (err != MPI_SUCCESS)
@@ -423,18 +284,10 @@ type_create_subarray64(int               ndims,
     if (err != MPI_SUCCESS)
         return ncmpii_error_mpi2nc(err, "MPI_Type_create_resized");
     MPI_Type_free(&type2);
-#else
-    MPI_Datatype types[3];
-    types[0] = MPI_LB;
-    types[1] = type1;
-    types[2] = MPI_UB;
-    err = MPI_Type_struct(3, blklens, disps, types, newtype);
-    if (err != MPI_SUCCESS)
-        return ncmpii_error_mpi2nc(err, "MPI_Type_struct");
-#endif
     MPI_Type_free(&type1);
 
     return NC_NOERR;
+#endif
 }
 
 /*----< filetype_create_vara() >--------------------------------------------*/
@@ -476,7 +329,11 @@ filetype_create_vara(const NC         *ncp,
 
     /* previously, request size has been checked and it must > 0 */
     if (IS_RECVAR(varp)) {
+#ifdef HAVE_MPI_LARGE_COUNT
+        MPI_Count blocklength;
+#else
         int blocklength;
+#endif
         MPI_Datatype rectype=MPI_BYTE;
 
 #if SIZEOF_MPI_AINT != SIZEOF_MPI_OFFSET
@@ -484,9 +341,6 @@ filetype_create_vara(const NC         *ncp,
         status = check_recsize_too_big(ncp->recsize);
         if (status != NC_NOERR) return status;
 #endif
-        /* check overflow, because 1st argument of hvector is of type int */
-        if (count[0] != (int) count[0]) DEBUG_RETURN_ERROR(NC_EINTOVERFLOW)
-
         offset += start[0] * ncp->recsize;
 
         if (varp->ndims > 1) {
@@ -504,18 +358,23 @@ filetype_create_vara(const NC         *ncp,
             blocklength = varp->xsz;
         }
 
+#ifdef HAVE_MPI_LARGE_COUNT
+        /* concatenate number of count[0] subarray types into filetype */
+        err = MPI_Type_create_hvector_c(count[0], blocklength, ncp->recsize,
+                                        rectype, &filetype);
+        if (err != MPI_SUCCESS)
+            return ncmpii_error_mpi2nc(err, "MPI_Type_create_hvector_c");
+#else
+        /* check overflow, because 1st argument of hvector is of type int */
+        if (count[0] > NC_MAX_INT) DEBUG_RETURN_ERROR(NC_EINTOVERFLOW)
+
         /* concatenate number of count[0] subarray types into filetype */
-#ifdef HAVE_MPI_TYPE_CREATE_HVECTOR
         err = MPI_Type_create_hvector((int)count[0], blocklength, ncp->recsize,
                                       rectype, &filetype);
         if (err != MPI_SUCCESS)
             return ncmpii_error_mpi2nc(err, "MPI_Type_create_hvector");
-#else
-        err = MPI_Type_hvector((int)count[0], blocklength, ncp->recsize,
-                               rectype, &filetype);
-        if (err != MPI_SUCCESS)
-            return ncmpii_error_mpi2nc(err, "MPI_Type_hvector");
 #endif
+
         if (rectype != MPI_BYTE) MPI_Type_free(&rectype);
     }
     else { /* for non-record variable, just create a subarray datatype */
@@ -536,75 +395,96 @@ filetype_create_vara(const NC         *ncp,
  * blocklens[] and disps[]
  */
 static int
-stride_flatten(int               isRecVar, /* whether record variable */
-               int               ndim,     /* number of dimensions */
-               int               el_size,  /* array element size */
-               const MPI_Offset *dimlen,   /* [ndim] dimension lengths */
-               const MPI_Offset *start,    /* [ndim] starts of subarray */
-               const MPI_Offset *count,    /* [ndim] counts of subarray */
-               const MPI_Offset *stride,   /* [ndim] strides of subarray */
-               int              *nblocks,  /* OUT: number of blocks */
-               int              *blocklens,/* OUT: length of each block */
-               MPI_Aint         *disps)    /* OUT: displacement of each block */
+stride_flatten(const NC_var      *varp,
+               MPI_Offset         recsize,  /* record size */
+               const MPI_Offset  *start,    /* [ndim] starts of subarray */
+               const MPI_Offset  *count,    /* [ndim] counts of subarray */
+               const MPI_Offset  *stride,   /* [ndim] strides of subarray */
+               MPI_Offset        *nblocks,  /* OUT: number of blocks */
+               MPI_Offset       **blocklens,/* OUT: length of each block */
+               MPI_Aint         **disps)    /* OUT: displacement of each block */
 {
-    int i, j, k, seg_len;
-    MPI_Offset nstride, array_len, off, subarray_len;
+    int i, j, k, isRecVar, ndims, el_size;
+    MPI_Offset *dimlen, seg_len, nstride, array_len, off, subarray_len;
+
+    *blocklens = NULL;
+    *disps = NULL;
+
+    /* whether record variable */
+    isRecVar = IS_RECVAR(varp);
+
+    ndims = varp->ndims;
 
     /* scalar variables have been handled before this subroutine is called */
-    assert (ndim > 0);
+    assert (ndims > 0);
+
+    /* array element size */
+    el_size = varp->xsz;
+
+    /* dimension lengths */
+    dimlen = (MPI_Offset*) NCI_Malloc(sizeof(MPI_Offset) * ndims);
+
+    for (i=0; i<ndims; i++) dimlen[i] = varp->shape[i];
+    /* for record variable, set dimlen[0] to the record size */
+    if (isRecVar) dimlen[0] = recsize;
 
     /* calculate the number of offset-length pairs */
-    *nblocks = (stride[ndim-1] == 1) ? 1 : count[ndim-1];
-    for (i=0; i<ndim-1; i++) *nblocks *= count[i];
+    *nblocks = (stride[ndims-1] == 1) ? 1 : count[ndims-1];
+    for (i=0; i<ndims-1; i++) *nblocks *= count[i];
     if (*nblocks == 0) return 1;
 
-    /* the length of all contiguous segments are of the same size */
-    seg_len  = (stride[ndim-1] == 1) ? count[ndim-1] : 1;
-    seg_len *= el_size;
-    nstride  = (stride[ndim-1] == 1) ? 1 : count[ndim-1];
+    *blocklens = (MPI_Offset*) NCI_Malloc(sizeof(MPI_Offset) * (*nblocks));
+    *disps = (MPI_Aint*) NCI_Malloc(sizeof(MPI_Aint) * (*nblocks));
 
-    for (i=0; i<*nblocks; i++) blocklens[i] = seg_len;
+    /* the length of all contiguous segments are of the same size */
+    seg_len  = (stride[ndims-1] == 1) ? count[ndims-1] : 1;
+    seg_len *= el_size;
+    nstride  = (stride[ndims-1] == 1) ? 1 : count[ndims-1];
+
+    for (i=0; i<*nblocks; i++) (*blocklens)[i] = seg_len;
 
     /* set the offset-length pairs for the lowest dimension */
-    disps[0] = start[ndim-1] * el_size;
+    (*disps)[0] = start[ndims-1] * el_size;
     for (k=1; k<nstride; k++)
-        disps[k] = disps[k-1] + stride[ndim-1] * el_size;
+        (*disps)[k] = (*disps)[k-1] + stride[ndims-1] * el_size;
 
     /* done with the lowest dimension */
-    ndim--;
+    ndims--;
 
     subarray_len = nstride;
     array_len = el_size;
     /* for higher dimensions */
-    while (ndim > 0) {
-        /* array_len is global array size from lowest up to ndim */
-        array_len *= dimlen[ndim];
+    while (ndims > 0) {
+        /* array_len is global array size from lowest up to ndims */
+        array_len *= dimlen[ndims];
 
-        /* off is the global array offset for this dimension, ndim-1
+        /* off is the global array offset for this dimension, ndims-1
          * For record variable, dimlen[0] is the sum of single record sizes
          */
-        if (ndim == 1 && isRecVar) off = start[0] * dimlen[0];
-        else off = start[ndim-1] * array_len;
+        if (ndims == 1 && isRecVar) off = start[0] * dimlen[0];
+        else off = start[ndims-1] * array_len;
 
-        /* update all offsets from lowest up to dimension ndim-1 */
-        for (j=0; j<subarray_len; j++) disps[j] += off;
+        /* update all offsets from lowest up to dimension ndims-1 */
+        for (j=0; j<subarray_len; j++) (*disps)[j] += off;
 
-        /* update each successive subarray of dimension ndim-1
+        /* update each successive subarray of dimension ndims-1
          * For record variable, dimlen[0] is the sum of single record sizes
          */
-        if (ndim == 1 && isRecVar) off = stride[0] * dimlen[0];
-        else off = stride[ndim-1] * array_len;
+        if (ndims == 1 && isRecVar) off = stride[0] * dimlen[0];
+        else off = stride[ndims-1] * array_len;
 
-        for (i=1; i<count[ndim-1]; i++) {
+        for (i=1; i<count[ndims-1]; i++) {
             for (j=0; j<subarray_len; j++)
-                disps[k++] = disps[j] + off;
+                (*disps)[k++] = (*disps)[j] + off;
 
-            if (ndim == 1 && isRecVar) off += stride[0] * dimlen[0];
-            else off += stride[ndim-1] * array_len;
+            if (ndims == 1 && isRecVar) off += stride[0] * dimlen[0];
+            else off += stride[ndims-1] * array_len;
         }
-        ndim--;  /* move to next higher dimension */
-        subarray_len *= count[ndim];
+        ndims--;  /* move to next higher dimension */
+        subarray_len *= count[ndims];
     }
+    NCI_Free(dimlen);
+
     return 1;
 }
 
@@ -619,9 +499,9 @@ ncmpio_filetype_create_vars(const NC         *ncp,
                             MPI_Datatype     *filetype_ptr,       /* OUT */
                             int              *is_filetype_contig) /* OUT */
 {
-    int           dim, err, nblocks, *blocklens;
+    int           err=NC_NOERR, mpireturn, dim, isLargeReq;
     MPI_Aint     *disps;
-    MPI_Offset    offset, nelems, *shape;
+    MPI_Offset    i, nblocks, nelems, *blocklens;
     MPI_Datatype  filetype=MPI_BYTE;
 
     if (stride == NULL)
@@ -643,11 +523,13 @@ ncmpio_filetype_create_vars(const NC         *ncp,
     nelems = 1;
     for (dim=0; dim<varp->ndims; dim++) nelems *= count[dim];
 
+    /* starting file offset of the variable */
+    *offset_ptr = varp->begin;
+
     /* when nelems == 0 or varp is a scalar, i.e. varp->ndims == 0, no need to
      * create a filetype
      */
     if (varp->ndims == 0 || nelems == 0) {
-        *offset_ptr   = varp->begin;
         *filetype_ptr = MPI_BYTE;
         if (is_filetype_contig != NULL) *is_filetype_contig = 1;
         return NC_NOERR;
@@ -655,149 +537,71 @@ ncmpio_filetype_create_vars(const NC         *ncp,
 
     /* hereinafter fileview is noncontiguous, i.e. filetype != MPI_BYTE */
     if (is_filetype_contig != NULL) *is_filetype_contig = 0;
-    offset = varp->begin;
-
-#if 1
-    blocklens = (int*) NCI_Malloc((size_t)nelems * SIZEOF_INT);
-    disps = (MPI_Aint*) NCI_Malloc((size_t)nelems * SIZEOF_MPI_AINT);
-    shape = (MPI_Offset*) NCI_Malloc((size_t)varp->ndims * SIZEOF_MPI_OFFSET);
-
-    for (dim=0; dim<varp->ndims; dim++) shape[dim] = varp->shape[dim];
-    /* for record variable, set shape[0] to the record size */
-    if (IS_RECVAR(varp)) shape[0] = ncp->recsize;
 
     /* flatten stride access into list of offsets and lengths stored in
      * disps[] and blocklens[], respectively.
      */
-    stride_flatten(IS_RECVAR(varp), varp->ndims, varp->xsz, shape, start,
-                   count, stride, &nblocks, blocklens, disps);
-    NCI_Free(shape);
+    stride_flatten(varp, ncp->recsize, start, count, stride, &nblocks,
+                   &blocklens, &disps);
 
     /* the flattened list allows one single call to hindexed constructor.
      * We cannot use MPI_Type_indexed because displacement for the record
      * dimension may not be a multiple of varp->xtype
      */
-    err = MPI_Type_create_hindexed(nblocks, blocklens, disps, MPI_BYTE,
-                                   &filetype);
-    NCI_Free(disps);
-    NCI_Free(blocklens);
-    if (err != MPI_SUCCESS)
-        return ncmpii_error_mpi2nc(err, "MPI_Type_create_hindexed");
-
-    MPI_Type_commit(&filetype);
-
-    /* Below is an old implementation that uses a nested hinexed constructor
-     * which can MPI error on depth beyond  DLOOP_MAX_DATATYPE_DEPTH.
-     * The above approach avoids such problem by flattening stride access into
-     * offset-length and using a single call to hindexed constructor.
-     */
-#else
-    int ndims, *blockcounts;
-    MPI_Aint *blockstride;
-    MPI_Offset   stride_off;
-    MPI_Datatype tmptype;
-
-    ndims       = varp->ndims;
-    blockcounts = (int*) NCI_Malloc((size_t)ndims * 2 * SIZEOF_INT);
-    blocklens   = blockcounts + ndims;
-
-    blockstride = (MPI_Aint*) NCI_Malloc((size_t)ndims * SIZEOF_MPI_AINT);
-
-    tmptype = MPI_BYTE;
-
-    blockcounts[ndims-1] = (int)count[ndims-1];
-    /* check 4-byte integer overflow (blockcounts in MPI_Type_hvector
-       is of type int while count[] is of type MPI_Offset */
-    if (count[ndims-1] != blockcounts[ndims-1]) {
-        NCI_Free(blockstride);
-        NCI_Free(blockcounts);
-        DEBUG_RETURN_ERROR(NC_EINTOVERFLOW)
-    }
-    /* blocklens[] is unlikely a big value */
-    blocklens[ndims-1] = varp->xsz;
-
-    if (ndims == 1 && IS_RECVAR(varp)) {
-#if SIZEOF_MPI_AINT != SIZEOF_MPI_OFFSET
-        err = check_recsize_too_big(ncp->recsize);
-        if (err != NC_NOERR) return err;
-#endif
-        stride_off = stride[ndims-1] * ncp->recsize;
-        blockstride[ndims-1] = stride_off;
-        offset += start[ndims-1] * ncp->recsize;
-    } else {
-        stride_off = stride[ndims-1] * varp->xsz;
-        blockstride[ndims-1] = stride_off;
-        offset += start[ndims-1] * varp->xsz;
-    }
-#if SIZEOF_MPI_AINT != SIZEOF_MPI_OFFSET
-    /* check 4-byte integer overflow */
-    if (stride_off != blockstride[ndims-1]) {
-        if (blockstride) NCI_Free(blockstride);
-        if (blockcounts) NCI_Free(blockcounts);
-        DEBUG_RETURN_ERROR(NC_EINTOVERFLOW)
-    }
-#endif
-
-    for (dim=ndims-1; dim>=0; dim--) {
-#ifdef HAVE_MPI_TYPE_CREATE_HVECTOR
-        err = MPI_Type_create_hvector(blockcounts[dim], blocklens[dim],
-                                      blockstride[dim], tmptype, &filetype);
-        if (err != MPI_SUCCESS) {
-            NCI_Free(blockstride);
-            NCI_Free(blockcounts);
-            return ncmpii_error_mpi2nc(err, "MPI_Type_create_hvector");
+    isLargeReq = 0;
+    if (nblocks > NC_MAX_INT)
+        isLargeReq = 1;
+    else {
+        for (i=0; i<nblocks; i++) {
+            if (blocklens[i] > NC_MAX_INT) {
+                isLargeReq = 1;
+                break;
+            }
         }
-#else
-        err = MPI_Type_hvector(blockcounts[dim], blocklens[dim],
-                               blockstride[dim], tmptype, &filetype);
-        if (err != MPI_SUCCESS) {
-            NCI_Free(blockstride);
-            NCI_Free(blockcounts);
-            return ncmpii_error_mpi2nc(err, "MPI_Type_hvector");
+    }
+
+    if (isLargeReq) {
+#ifdef HAVE_MPI_TYPE_CREATE_HINDEXED_C
+        MPI_Count *blocklens_c, *disps_c;
+        blocklens_c = (MPI_Count*) NCI_Malloc(sizeof(MPI_Count) * nelems);
+        disps_c     = (MPI_Count*) NCI_Malloc(sizeof(MPI_Count) * nelems);
+        for (i=0; i<nelems; i++) {
+            blocklens_c[i] = blocklens[i];
+            disps_c[i] = disps[i];
         }
+        mpireturn = MPI_Type_create_hindexed_c(nblocks, blocklens_c, disps_c,
+                                               MPI_BYTE, &filetype);
+        NCI_Free(blocklens_c);
+        NCI_Free(disps_c);
+#else
+        *filetype_ptr = MPI_DATATYPE_NULL;
+        DEBUG_ASSIGN_ERROR(err, NC_EINTOVERFLOW)
+        mpireturn = MPI_SUCCESS;
 #endif
+    }
+    else {
+        int *blocklens_i;
+        blocklens_i = (int*) NCI_Malloc(sizeof(int) * nelems);
+        for (i=0; i<nelems; i++)
+            blocklens_i[i] = (int)blocklens[i];
+        mpireturn = MPI_Type_create_hindexed((int)nblocks, blocklens_i, disps,
+                                             MPI_BYTE, &filetype);
+        NCI_Free(blocklens_i);
+    }
+
+    if (disps != NULL) NCI_Free(disps);
+    if (blocklens != NULL) NCI_Free(blocklens);
+
+    if (mpireturn != MPI_SUCCESS) {
+        err = ncmpii_error_mpi2nc(mpireturn, "MPI_Type_create_hindexed");
+        *filetype_ptr = MPI_DATATYPE_NULL;
+    }
+    else {
         MPI_Type_commit(&filetype);
-        if (tmptype != MPI_BYTE)
-            MPI_Type_free(&tmptype);
-        tmptype = filetype;
-
-        if (dim - 1 >= 0) {
-            blocklens[dim-1]  = 1;
-            blockcounts[dim-1] = (int)count[dim - 1];
-            /* check 4-byte integer overflow */
-            if (count[dim-1] != blockcounts[dim-1]) {
-                NCI_Free(blockstride);
-                NCI_Free(blockcounts);
-                DEBUG_RETURN_ERROR(NC_EINTOVERFLOW)
-            }
-
-            if (dim-1 == 0 && IS_RECVAR(varp)) {
-                stride_off = stride[dim-1] * ncp->recsize;
-                blockstride[dim-1] = stride_off;
-                offset += start[dim-1] * ncp->recsize;
-            } else {
-                stride_off = stride[dim-1] * varp->dsizes[dim] * varp->xsz;
-                blockstride[dim-1] = stride_off;
-                offset += start[dim-1] * varp->dsizes[dim] * varp->xsz;
-            }
-#if SIZEOF_MPI_AINT != SIZEOF_MPI_OFFSET
-            /* check 4-byte integer overflow */
-            if (stride_off != blockstride[dim-1]) {
-                NCI_Free(blockstride);
-                NCI_Free(blockcounts);
-                DEBUG_RETURN_ERROR(NC_EINTOVERFLOW)
-            }
-#endif
-        }
+        *filetype_ptr = filetype;
     }
-    NCI_Free(blockstride);
-    NCI_Free(blockcounts);
-#endif
 
-    *offset_ptr   = offset;
-    *filetype_ptr = filetype;
-
-    return NC_NOERR;
+    return err;
 }
 
 /*----< ncmpio_file_set_view() >---------------------------------------------*/
@@ -826,41 +630,60 @@ ncmpio_file_set_view(const NC     *ncp,
     MPI_Comm_rank(ncp->comm, &rank);
     if (rank == 0) {
         /* prepend the whole file header to filetype */
+        MPI_Datatype root_filetype=MPI_BYTE, ftypes[2];
+#ifdef HAVE_MPI_LARGE_COUNT
+        MPI_Count blocklens[2];
+        MPI_Count disps[2];
+        blocklens[0] = ncp->begin_var;
+#else
         int blocklens[2];
         MPI_Aint disps[2];
-        MPI_Datatype root_filetype, ftypes[2];
-
-        /* first block is the header extent */
-        blocklens[0] = (int)ncp->begin_var;
-            disps[0] = 0;
-           ftypes[0] = MPI_BYTE;
 
         /* check if header size > 2^31 */
-        if (ncp->begin_var > INT_MAX)
-            DEBUG_ASSIGN_ERROR(status, NC_EINTOVERFLOW)
+        if (ncp->begin_var > NC_MAX_INT) {
+            DEBUG_ASSIGN_ERROR(status, NC_EINTOVERFLOW);
+            goto err_out;
+        }
+
+        blocklens[0] = (int)ncp->begin_var;
+#endif
+
+        /* first block is the header extent */
+            disps[0] = 0;
+           ftypes[0] = MPI_BYTE;
 
         /* second block is filetype, the subarray request(s) to the variable */
         blocklens[1] = 1;
             disps[1] = *offset;
            ftypes[1] = filetype;
 
-#if SIZEOF_MPI_AINT != SIZEOF_MPI_OFFSET
-        if (*offset > INT_MAX) {
-            blocklens[1] = 0;
-            DEBUG_ASSIGN_ERROR(status, NC_EAINT_TOO_SMALL)
+#if !defined(HAVE_MPI_LARGE_COUNT) && (SIZEOF_MPI_AINT != SIZEOF_MPI_OFFSET)
+        if (*offset > NC_MAX_INT) {
+            DEBUG_ASSIGN_ERROR(status, NC_EINTOVERFLOW);
+            goto err_out;
         }
 #endif
 
-#ifdef HAVE_MPI_TYPE_CREATE_STRUCT
-        MPI_Type_create_struct(2, blocklens, disps, ftypes, &root_filetype);
+#ifdef HAVE_MPI_LARGE_COUNT
+        mpireturn = MPI_Type_create_struct_c(2, blocklens, disps, ftypes,
+                                             &root_filetype);
 #else
-        MPI_Type_struct(2, blocklens, disps, ftypes, &root_filetype);
+        mpireturn = MPI_Type_create_struct(2, blocklens, disps, ftypes,
+                                           &root_filetype);
 #endif
+        if (mpireturn != MPI_SUCCESS) {
+            err = ncmpii_error_mpi2nc(mpireturn, "MPI_Type_create_struct");
+            if (status == NC_NOERR) status = err;
+        }
         MPI_Type_commit(&root_filetype);
 
+#ifndef HAVE_MPI_LARGE_COUNT
+err_out:
+#endif
         TRACE_IO(MPI_File_set_view)(fh, 0, MPI_BYTE, root_filetype, "native",
                                     MPI_INFO_NULL);
-        MPI_Type_free(&root_filetype);
+        if (root_filetype != MPI_BYTE)
+            MPI_Type_free(&root_filetype);
 
         /* now update the explicit offset to be used in MPI-IO call later */
         *offset = ncp->begin_var;
