@@ -54,7 +54,7 @@ ncmpio_free_NC_var(NC_var *varp)
 
 /*----< ncmpio_new_NC_var() >------------------------------------------------*/
 NC_var *
-ncmpio_new_NC_var(char *name, int ndims)
+ncmpio_new_NC_var(char *name, size_t name_len, int ndims)
 {
     NC_var *varp;
 
@@ -68,7 +68,7 @@ ncmpio_new_NC_var(char *name, int ndims)
     }
 
     varp->name     = name;         /* name has been malloc-ed */
-    varp->name_len = strlen(name); /* name has been NULL checked */
+    varp->name_len = name_len;     /* name has been NULL checked */
     varp->ndims    = ndims;
 
     return varp;
@@ -76,7 +76,7 @@ ncmpio_new_NC_var(char *name, int ndims)
 
 /*----< dup_NC_var() >-------------------------------------------------------*/
 static NC_var *
-dup_NC_var(const NC_var *rvarp)
+dup_NC_var(const NC_var *rvarp, int attr_hsize)
 {
     char *name;
     NC_var *varp;
@@ -87,7 +87,7 @@ dup_NC_var(const NC_var *rvarp)
     strcpy(name, rvarp->name);
 
     /* allocate a NC_var object */
-    varp = ncmpio_new_NC_var(name, rvarp->ndims);
+    varp = ncmpio_new_NC_var(name, rvarp->name_len, rvarp->ndims);
     if (varp == NULL ) return NULL;
 
     varp->xtype = rvarp->xtype;
@@ -96,6 +96,12 @@ dup_NC_var(const NC_var *rvarp)
     if (rvarp->ndims != 0 && rvarp->dimids != NULL)
         memcpy(varp->dimids, rvarp->dimids, (size_t)rvarp->ndims * SIZEOF_INT);
 
+#ifndef SEARCH_NAME_LINEARLY
+    /* initialize hashing lookup table size for attributes */
+    varp->attrs.hash_size = attr_hsize;
+    varp->attrs.nameT = NULL;
+#endif
+
     /* copy attributes */
     if (ncmpio_dup_NC_attrarray(&varp->attrs, &rvarp->attrs) != NC_NOERR) {
         ncmpio_free_NC_var(varp);
@@ -105,8 +111,10 @@ dup_NC_var(const NC_var *rvarp)
     /* copy the contents of shape may not be necessary, as one must call
      * compute_var_shape() to recompute it after a new variable is created
      */
-    memcpy(varp->shape,  rvarp->shape,  (size_t)rvarp->ndims * SIZEOF_MPI_OFFSET);
-    memcpy(varp->dsizes, rvarp->dsizes, (size_t)rvarp->ndims * SIZEOF_MPI_OFFSET);
+    if (rvarp->ndims > 0) { /* skip memcpy if ndims == 0 (scalar variables) */
+        memcpy(varp->shape,  rvarp->shape,  (size_t)rvarp->ndims * SIZEOF_MPI_OFFSET);
+        memcpy(varp->dsizes, rvarp->dsizes, (size_t)rvarp->ndims * SIZEOF_MPI_OFFSET);
+    }
     varp->xsz   = rvarp->xsz;
     varp->len   = rvarp->len;
     varp->begin = rvarp->begin;
@@ -123,7 +131,6 @@ ncmpio_free_NC_vararray(NC_vararray *ncap)
     int i;
 
     assert(ncap != NULL);
-    if (ncap->ndefined == 0) return;
 
     if (ncap->value != NULL) {
         /* when error is detected reading NC_VARIABLE tag, ncap->ndefined can
@@ -140,16 +147,23 @@ ncmpio_free_NC_vararray(NC_vararray *ncap)
 
 #ifndef SEARCH_NAME_LINEARLY
     /* free space allocated for var name lookup table */
-    ncmpio_hash_table_free(ncap->nameT);
+    if (ncap->nameT != NULL) {
+        ncmpio_hash_table_free(ncap->nameT, ncap->hash_size);
+        NCI_Free(ncap->nameT);
+        ncap->nameT = NULL;
+        ncap->hash_size = 0;
+    }
 #endif
 }
 
 /*----< ncmpio_dup_NC_vararray() >-------------------------------------------*/
 int
 ncmpio_dup_NC_vararray(NC_vararray       *ncap,
-                       const NC_vararray *ref)
+                       const NC_vararray *ref,
+                       int                attr_hsize)
 {
     int i, status=NC_NOERR;
+    size_t alloc_size;
 
     assert(ref != NULL);
     assert(ncap != NULL);
@@ -160,16 +174,14 @@ ncmpio_dup_NC_vararray(NC_vararray       *ncap,
         return NC_NOERR;
     }
 
-    if (ref->ndefined > 0) {
-        size_t alloc_size = _RNDUP(ref->ndefined, NC_ARRAY_GROWBY);
-        ncap->value = (NC_var **) NCI_Calloc(alloc_size, sizeof(NC_var*));
-        if (ncap->value == NULL) DEBUG_RETURN_ERROR(NC_ENOMEM)
-    }
+    alloc_size = _RNDUP(ref->ndefined, PNC_ARRAY_GROWBY);
+    ncap->value = (NC_var **) NCI_Calloc(alloc_size, sizeof(NC_var*));
+    if (ncap->value == NULL) DEBUG_RETURN_ERROR(NC_ENOMEM)
 
     /* duplicate one NC_var object at a time */
     ncap->ndefined = 0;
     for (i=0; i<ref->ndefined; i++) {
-        ncap->value[i] = dup_NC_var(ref->value[i]);
+        ncap->value[i] = dup_NC_var(ref->value[i], attr_hsize);
         if (ncap->value[i] == NULL) {
             DEBUG_ASSIGN_ERROR(status, NC_ENOMEM)
             break;
@@ -183,8 +195,12 @@ ncmpio_dup_NC_vararray(NC_vararray       *ncap,
     assert(ncap->ndefined == ref->ndefined);
 
 #ifndef SEARCH_NAME_LINEARLY
+    /* allocate hashing lookup table, if not allocated yet */
+    if (ncap->nameT == NULL)
+        ncap->nameT = NCI_Calloc(ncap->hash_size, sizeof(NC_nametable));
+
     /* duplicate var name lookup table */
-    ncmpio_hash_table_copy(ncap->nameT, ref->nameT);
+    ncmpio_hash_table_copy(ncap->nameT, ref->nameT, ncap->hash_size);
 #endif
 
     return NC_NOERR;
@@ -210,7 +226,8 @@ NC_findvar(const NC_vararray *ncap,
 
     assert (ncap != NULL);
 
-    if (ncap->ndefined == 0) return NC_ENOTVAR;
+    if (ncap->ndefined == 0)
+        DEBUG_RETURN_ERROR(NC_ENOTVAR)
 
     nchars = strlen(name);
     for (varid=0; varid<ncap->ndefined; varid++) {
@@ -221,7 +238,7 @@ NC_findvar(const NC_vararray *ncap,
         }
     }
 
-    return NC_ENOTVAR; /* not found */
+    DEBUG_RETURN_ERROR(NC_ENOTVAR) /* not found */
 }
 #else
 /*----< NC_findvar() >-------------------------------------------------------*/
@@ -238,10 +255,11 @@ NC_findvar(const NC_vararray  *ncap,
 
     assert (ncap != NULL);
 
-    if (ncap->ndefined == 0) return NC_ENOTVAR;
+    if (ncap->ndefined == 0)
+        DEBUG_RETURN_ERROR(NC_ENOTVAR)
 
     /* hash the var name into a key for name lookup */
-    key = HASH_FUNC(name);
+    key = HASH_FUNC(name, ncap->hash_size);
 
     /* check the list using linear search */
     nchars = strlen(name);
@@ -254,7 +272,7 @@ NC_findvar(const NC_vararray  *ncap,
         }
     }
 
-    return NC_ENOTVAR; /* the name has never been used */
+    DEBUG_RETURN_ERROR(NC_ENOTVAR) /* the name has never been used */
 }
 #endif
 
@@ -310,7 +328,8 @@ ncmpio_NC_var_shape64(NC_var            *varp,
 
 out :
     /* No variable size can be > X_INT64_MAX - 3 */
-    if (0 == ncmpio_NC_check_vlen(varp, X_INT64_MAX-3)) return NC_EVARSIZE;
+    if (0 == ncmpio_NC_check_vlen(varp, X_INT64_MAX-3))
+        DEBUG_RETURN_ERROR(NC_EVARSIZE)
 
     /*
      * For CDF-1 and CDF-2 formats, the total number of array elements
@@ -354,7 +373,7 @@ ncmpio_def_var(void       *ncdp,
     if (err != NC_NOERR) goto err_check;
 
     /* allocate a new NC_var object */
-    varp = ncmpio_new_NC_var(nname, ndims);
+    varp = ncmpio_new_NC_var(nname, strlen(nname), ndims);
     if (varp == NULL) {
         DEBUG_ASSIGN_ERROR(err, NC_ENOMEM)
         goto err_check;
@@ -376,8 +395,8 @@ ncmpio_def_var(void       *ncdp,
     }
 
     /* allocate/expand ncp->vars.value array */
-    if (ncp->vars.ndefined % NC_ARRAY_GROWBY == 0) {
-        size_t alloc_size = (size_t)ncp->vars.ndefined + NC_ARRAY_GROWBY;
+    if (ncp->vars.ndefined % PNC_ARRAY_GROWBY == 0) {
+        size_t alloc_size = (size_t)ncp->vars.ndefined + PNC_ARRAY_GROWBY;
         ncp->vars.value = (NC_var **) NCI_Realloc(ncp->vars.value,
                                       alloc_size * sizeof(NC_var*));
         if (ncp->vars.value == NULL) {
@@ -419,8 +438,14 @@ err_check:
     assert(nname != NULL);
 
 #ifndef SEARCH_NAME_LINEARLY
+    varp->attrs.hash_size = ncp->hash_size_attr;
+
+    /* allocate hashing lookup table, if not allocated yet */
+    if (ncp->vars.nameT == NULL)
+        ncp->vars.nameT = NCI_Calloc(ncp->vars.hash_size, sizeof(NC_nametable));
+
     /* insert nname to the lookup table */
-    ncmpio_hash_insert(ncp->vars.nameT, nname, varp->varid);
+    ncmpio_hash_insert(ncp->vars.nameT, ncp->vars.hash_size, nname, varp->varid);
 #endif
 
     if (varidp != NULL) *varidp = varp->varid;
@@ -507,13 +532,14 @@ ncmpio_inq_var(void       *ncdp,
 #endif
             *ndimsp = varp->ndims;
     }
-    if (dimids != NULL) {
+    if (dimids != NULL) { /* copy dim IDs for non-scalar */
 #ifdef ENABLE_SUBFILING
         /* varp->dimids_org is already set during open or enddef */
-        if (varp->num_subfiles > 1)
+        if (varp->num_subfiles > 1 && varp->ndims_org > 0)
             memcpy(dimids, varp->dimids_org, (size_t)varp->ndims_org * SIZEOF_INT);
         else
 #endif
+        if (varp->ndims > 0)
             memcpy(dimids, varp->dimids, (size_t)varp->ndims * SIZEOF_INT);
     }
     if (nattsp != NULL) *nattsp = varp->attrs.ndefined;
@@ -565,8 +591,8 @@ ncmpio_rename_var(void       *ncdp,
 
 #ifndef SEARCH_NAME_LINEARLY
     /* update var name lookup table */
-    err = ncmpio_update_name_lookup_table(ncp->vars.nameT, varid,
-                        ncp->vars.value[varid]->name, nnewname);
+    err = ncmpio_update_name_lookup_table(ncp->vars.nameT, ncp->vars.hash_size,
+                        varid, ncp->vars.value[varid]->name, nnewname);
     if (err != NC_NOERR) {
         DEBUG_TRACE_ERROR(err)
         goto err_check;
