@@ -22,7 +22,6 @@
 #include <stdlib.h>
 #endif
 #include <string.h> /* memcpy() */
-#include <limits.h> /* INT_MAX */
 #include <assert.h>
 
 #include <mpi.h>
@@ -56,17 +55,11 @@ getput_vard(NC               *ncp,
     void *xbuf=NULL;
     int mpireturn, status=NC_NOERR, err=NC_NOERR, xtype_is_contig=1;
     int el_size, buftype_is_contig=0, need_swap_back_buf=0;
-    int nelems=0, need_convert=0, need_swap=0, coll_indep, rw_flag;
+    int need_convert=0, need_swap=0, coll_indep, rw_flag;
     MPI_File fh;
-    MPI_Offset fnelems=0, bnelems=0, offset=0;
+    MPI_Offset nelems=0, fnelems=0, bnelems=0, offset=0;
     MPI_Datatype etype=MPI_DATATYPE_NULL, xtype=MPI_BYTE;
-#if MPI_VERSION >= 3
-    MPI_Count filetype_size=0;
-    MPI_Count true_lb=0, true_ub=0, true_extent=0;
-#else
-    int filetype_size=0;
-    MPI_Aint true_lb=0, true_ub=0, true_extent=0;
-#endif
+    MPI_Offset filetype_size;
 
 #ifdef ENABLE_SUBFILING
     /* call a separate routine if variable is stored in subfiles */
@@ -91,47 +84,51 @@ getput_vard(NC               *ncp,
      * MPI_Type_create_hindexed), we need to find the true last byte accessed
      * by this request, true_ub, in order to calculate new_numrecs.
      */
-#if MPI_VERSION >= 3
+#ifdef HAVE_MPI_TYPE_SIZE_C
+    /* MPI_Type_size_c is introduced in MPI 4.0 */
+    MPI_Count true_lb=0, true_ub=0, true_extent=0;
+    MPI_Count type_size;
+
+    mpireturn = MPI_Type_size_c(filetype, &type_size);
+#elif defined(HAVE_MPI_TYPE_SIZE_X)
     /* MPI_Type_size_x is introduced in MPI 3.0 */
-    mpireturn = MPI_Type_size_x(filetype, &filetype_size);
-    if (mpireturn != MPI_SUCCESS) {
-        err = ncmpii_error_mpi2nc(mpireturn, "MPI_Type_size_x");
-        xtype = MPI_BYTE;
-        goto err_check;
-    }
-    /* MPI_Type_get_true_extent_x is introduced in MPI 3.0 */
-    MPI_Type_get_true_extent_x(filetype, &true_lb, &true_extent);
-    true_ub = true_lb + true_extent;
+    MPI_Count true_lb=0, true_ub=0, true_extent=0;
+    MPI_Count type_size;
+
+    mpireturn = MPI_Type_size_x(filetype, &type_size);
 #else
     /* PROBLEM: In MPI_Type_size(), argument filetype_size is a 4-byte integer,
      * cannot be used for large filetypes. Prior to MPI 3.0 standard, argument
      * "size" of MPI_Type_size is of type int. When int overflows, the returned
      * value in argument "size" may be a negative. */
-    mpireturn = MPI_Type_size(filetype, &filetype_size);
+    MPI_Aint true_lb=0, true_ub=0, true_extent=0;
+    int type_size;
+
+    mpireturn = MPI_Type_size(filetype, &type_size);
+#endif
     if (mpireturn != MPI_SUCCESS) {
         err = ncmpii_error_mpi2nc(mpireturn, "MPI_Type_size");
-        xtype = MPI_BYTE;
         goto err_check;
     }
-    if (filetype_size < 0) { /* int overflow */
+    if (type_size == MPI_UNDEFINED) { /* int overflow */
         DEBUG_ASSIGN_ERROR(err, NC_EINTOVERFLOW)
         goto err_check;
     }
+    else if (type_size == 0) /* zero-length request */
+        goto err_check;
+
+    filetype_size = type_size;
+
+#ifdef HAVE_MPI_TYPE_GET_TRUE_EXTENT_C
+    /* MPI_Type_get_true_extent_c is introduced in MPI 4.0 */
+    MPI_Type_get_true_extent_c(filetype, &true_lb, &true_extent);
+#elif defined(HAVE_MPI_TYPE_GET_TRUE_EXTENT_X)
+    /* MPI_Type_get_true_extent_x is introduced in MPI 3.0 */
+    MPI_Type_get_true_extent_x(filetype, &true_lb, &true_extent);
+#else
     MPI_Type_get_true_extent(filetype, &true_lb, &true_extent);
+#endif
     true_ub = true_lb + true_extent;
-#endif
-
-    if (filetype_size == 0) { /* zero-length request */
-        goto err_check;
-    }
-
-#ifndef ENABLE_LARGE_SINGLE_REQ
-    /* Not all MPI-IO libraries support single requests larger than 2 GiB */
-    if (filetype_size > INT_MAX) {
-        DEBUG_ASSIGN_ERROR(err, NC_EINTOVERFLOW)
-        goto err_check;
-    }
-#endif
 
     /* get the corresponding MPI datatype of variable external type */
     xtype = ncmpii_nc2mpitype(varp->xtype);
@@ -174,12 +171,6 @@ getput_vard(NC               *ncp,
         if (err != NC_NOERR) goto err_check;
 
         bnelems *= bufcount;
-#ifndef ENABLE_LARGE_SINGLE_REQ
-        if (bnelems != (int)bnelems) {
-            DEBUG_ASSIGN_ERROR(err, NC_EINTOVERFLOW)
-            goto err_check;
-        }
-#endif
 
         /* filetype's number of elements must be equal to request's */
         if (fnelems != bnelems) {
@@ -252,7 +243,7 @@ getput_vard(NC               *ncp,
     /* Set nelems and xtype which will be used in MPI read/write */
     if (buf != xbuf) {
         /* xbuf is a malloc-ed contiguous buffer */
-        nelems = (int)bnelems;
+        nelems = bnelems;
     }
     else {
         /* we can safely use bufcount and buftype in MPI File read/write.
@@ -371,12 +362,6 @@ err_check:
                 }
             }
         }
-
-        if (NC_doFsync(ncp)) { /* NC_SHARE is set */
-            TRACE_IO(MPI_File_sync)(fh);
-            if (fIsSet(reqMode, NC_REQ_COLL))
-                TRACE_COMM(MPI_Barrier)(ncp->comm);
-        }
     }
 
     if (xbuf != NULL && xbuf != buf) NCI_Free(xbuf);
