@@ -582,10 +582,10 @@ do_ncdump(const char *path, struct fspec* specp)
                                 var.fillval = NC_FILL_DOUBLE;
                                 break;
                             case NC_INT64:
-                                var.fillval = NC_FILL_INT64;
+                                var.fillval = (double)NC_FILL_INT64;
                                 break;
                             case NC_UINT64:
-                                var.fillval = NC_FILL_UINT64;
+                                var.fillval = (double)NC_FILL_UINT64;
                                 break;
                             default:
                                 break;
@@ -743,7 +743,8 @@ enum FILE_KIND check_file_signature(char *path)
     char *cdf_signature="CDF";
     char *hdf5_signature="\211HDF\r\n\032\n";
     char signature[8];
-    int fd, rlen;
+    int fd;
+    size_t rlen;
 
     if ((fd = open(path, O_RDONLY, 0700)) == -1) {
         fprintf(stderr,"%s error at opening file %s (%s)\n",progname,path,strerror(errno));
