@@ -1499,7 +1499,11 @@ hdr_get_NC_var(bufferinfo  *gbp,
             return status;
         }
         if (gbp->version == 5)
-            varp->dimids[dim] = get_uint64(gbp);
+            /* number of dimensions defined in a NetCDF file is limited to
+             * NC_MAX_DIMS, which is NC_MAX_INT. Thus it is safe to cast to a
+             * 4-byte int.
+             */
+            varp->dimids[dim] = (int)get_uint64(gbp);
         else
             varp->dimids[dim] = get_uint32(gbp);
     }
@@ -1885,7 +1889,7 @@ usage(char *cmd)
 "       [-x]            Check gaps in fixed-size variables, output 1 if gaps\n"
 "                       are found, 0 for otherwise.\n"
 "       file            Input netCDF file name\n"
-"*PnetCDF library version 1.12.3 of February 21, 2022\n";
+"*PnetCDF library version PNETCDF_RELEASE_VERSION of PNETCDF_RELEASE_DATE\n";
     fprintf(stderr, help, cmd);
 }
 
@@ -2005,7 +2009,10 @@ int main(int argc, char *argv[])
 
     /* print file name and format */
     printf("netcdf %s {\n",filename);
-    printf("// file format: CDF-%d\n",ncp->flags);
+    printf("// File format: CDF-%d\n",ncp->flags);
+    printf("// Number of dimensions: %d\n",ncp->dims.ndefined);
+    printf("// Number of variables: %d\n",ncp->vars.ndefined);
+    printf("// Number of global attributes: %d\n",ncp->attrs.ndefined);
     printf("\n");
 
     /* print file header size and extent */
@@ -2080,14 +2087,14 @@ int main(int argc, char *argv[])
             printf("\t%6s %s:\n", type_str, varp->name->cp);
         } else {
             size *= varp->dsizes[0];
-            lineLen = strlen(varp->name->cp) + 2;
+            lineLen = varp->name->nchars + 2;
             ndims = MIN(varp->ndims, MAX_PRINT_NDIMS);
             for (j=0; j<ndims; j++) {
                 dimp = ncp->dims.value[varp->dimids[j]];
-                lineLen += strlen(dimp->name->cp) + 2;
+                lineLen += dimp->name->nchars + 2;
             }
             dimp = ncp->dims.value[varp->dimids[varp->ndims-1]];
-            lineLen += strlen(dimp->name->cp) + 2 + 5; /* ", " and "..., " */
+            lineLen += dimp->name->nchars + 2 + 5; /* ", " and "..., " */
             line = (char*)malloc(lineLen);
             sprintf(line,"%s(", varp->name->cp);
             ndims = varp->ndims;
@@ -2172,14 +2179,14 @@ int main(int argc, char *argv[])
             printf("\t%6s %s:", type_str, varp->name->cp);
         } else {
             size *= varp->dsizes[0];
-            lineLen = strlen(varp->name->cp) + 2;
+            lineLen = varp->name->nchars + 2;
             ndims = MIN(varp->ndims, MAX_PRINT_NDIMS);
             for (j=0; j<ndims; j++) {
                 dimp = ncp->dims.value[varp->dimids[j]];
-                lineLen += strlen(dimp->name->cp) + 2;
+                lineLen += dimp->name->nchars + 2;
             }
             dimp = ncp->dims.value[varp->dimids[varp->ndims-1]];
-            lineLen += strlen(dimp->name->cp) + 2 + 5; /* ", " and "..., " */
+            lineLen += dimp->name->nchars + 2 + 5; /* ", " and "..., " */
             line = (char*)malloc(lineLen);
             sprintf(line,"%s(", varp->name->cp);
             ndims = varp->ndims;
