@@ -6,10 +6,10 @@
 
 #include <stdio.h>
 #include <stdlib.h>
-#include <sys/types.h>  /* open() */
-#include <sys/stat.h>   /* open() */
+#include <sys/types.h>  /* open(), fstat() */
+#include <sys/stat.h>   /* open(), fstat() */
 #include <fcntl.h>      /* open() */
-#include <unistd.h>     /* read(), getopt() */
+#include <unistd.h>     /* read(), getopt(), fstat() */
 #include <string.h>     /* strcpy(), strncpy() */
 #include <inttypes.h>   /* check for Endianness, uint32_t*/
 #include <assert.h>
@@ -34,6 +34,12 @@
 static int verbose, trace;
 static int repair;
 static const char nada[4] = {0, 0, 0, 0};
+/*
+ * static const char *fmt_limit = "https://www.unidata.ucar.edu/software/netcdf/docs/file_structure_and_performance.html#classic_format_limitations";
+ * static const char *off_limit = "http://www.unidata.ucar.edu/software/netcdf/docs/file_structure_and_performance.html#offset_format_limitations";
+ */
+static const char *fmt_limit = "https://docs.unidata.ucar.edu/nug/current/file_structure_and_performance.html#classic_format_limitations";
+static const char *off_limit = "https://docs.unidata.ucar.edu/nug/current/file_structure_and_performance.html#offset_format_limitations";
 
 #ifndef MAX
 #define MAX(mm,nn) (((mm) > (nn)) ? (mm) : (nn))
@@ -971,6 +977,7 @@ static int
 hdr_get_name(int          fd,
              bufferinfo  *gbp,
              char       **namep,
+             size_t      *name_len,
              const char  *loc)
 {
     /* netCDF file format:
@@ -999,6 +1006,7 @@ hdr_get_name(int          fd,
         if (verbose) printf("\t%s: Failed to read name string length\n", loc);
         return err;
     }
+    *name_len = nchars;
 
     *namep = (char*) malloc((size_t)nchars + 1);
     if (*namep == NULL) DEBUG_RETURN_ERROR(NC_ENOMEM)
@@ -1064,13 +1072,13 @@ static int
 val_get_NC_dim(int fd, bufferinfo *gbp, NC_dim **dimpp, NC_dimarray *ncap) {
     int err, status=NC_NOERR;
     char *name=NULL;
-    size_t err_addr;
+    size_t err_addr, name_len;
     long long dim_length;
     NC_dim *dimp;
 
     *dimpp = NULL;
 
-    status = hdr_get_name(fd, gbp, &name, "Dimension");
+    status = hdr_get_name(fd, gbp, &name, &name_len, "Dimension");
     if (status != NC_NOERR && status != NC_ENULLPAD) {
         if (name != NULL) free(name);
         return status;
@@ -1100,7 +1108,7 @@ val_get_NC_dim(int fd, bufferinfo *gbp, NC_dim **dimpp, NC_dimarray *ncap) {
         DEBUG_RETURN_ERROR(NC_ENOMEM)
     }
     dimp->name     = name;
-    dimp->name_len = strlen(name);
+    dimp->name_len = name_len;
     dimp->size     = dim_length;
 
     *dimpp = dimp;
@@ -1355,6 +1363,7 @@ x_len_NC_attrV(nc_type    xtype,
 
 static int
 new_NC_attr(char        *name,
+            size_t       name_len,
             nc_type      xtype,
             long long   nelems,
             NC_attr    **attrp)
@@ -1367,7 +1376,7 @@ new_NC_attr(char        *name,
     (*attrp)->nelems   = nelems;
     (*attrp)->xvalue   = NULL;
     (*attrp)->name     = name;
-    (*attrp)->name_len = strlen(name);
+    (*attrp)->name_len = name_len;
 
     if (nelems > 0) {
         long long xsz = x_len_NC_attrV(xtype, nelems);
@@ -1390,12 +1399,12 @@ val_get_NC_attr(int          fd,
 {
     char *name=NULL, xloc[1024];
     int err, status=NC_NOERR;
-    size_t err_addr;
+    size_t err_addr, name_len;
     nc_type xtype;
     long long nelems;
     NC_attr *attrp;
 
-    status = hdr_get_name(fd, gbp, &name, loc);
+    status = hdr_get_name(fd, gbp, &name, &name_len, loc);
     if (status != NC_NOERR && status != NC_ENULLPAD) {
         if (name != NULL) free(name);
         return status;
@@ -1417,7 +1426,7 @@ val_get_NC_attr(int          fd,
         return err;
     }
 
-    err = new_NC_attr(name, xtype, nelems, &attrp);
+    err = new_NC_attr(name, name_len, xtype, nelems, &attrp);
     if(err != NC_NOERR) {
         if (name != NULL) free(name);
         return err;
@@ -1551,7 +1560,7 @@ val_get_NC_attrarray(int           fd,
 
 /*----< ncmpio_new_NC_var() >------------------------------------------------*/
 static NC_var *
-val_new_NC_var(char *name, int ndims)
+val_new_NC_var(char *name, size_t name_len, int ndims)
 {
     NC_var *varp;
 
@@ -1565,7 +1574,7 @@ val_new_NC_var(char *name, int ndims)
     }
 
     varp->name     = name;
-    varp->name_len = strlen(name);
+    varp->name_len = name_len;
     varp->ndims    = ndims;
     varp->xsz      = 0;
     varp->len      = 0;
@@ -1619,13 +1628,13 @@ val_get_NC_var(int          fd,
      *               <non-negative INT64>  // CDF-5
      */
     char *name=NULL, xloc[1024];
-    int dim, dimid, err, status=NC_NOERR;
-    size_t err_addr;
-    long long ndims;
+    int dim, dimid, ndims, err, status=NC_NOERR;
+    size_t err_addr, name_len;
+    long long ndims64;
     NC_var *varp;
 
     /* read variable name */
-    err = hdr_get_name(fd, gbp, &name, loc);
+    err = hdr_get_name(fd, gbp, &name, &name_len, loc);
     if (err != NC_NOERR && err != NC_ENULLPAD) {
         if (name != NULL) free(name);
         return err;
@@ -1637,7 +1646,7 @@ val_get_NC_var(int          fd,
     /* read number of dimensions */
     sprintf(xloc,"%s \"%s\"",loc,name);
     err_addr = ERR_ADDR;
-    err = hdr_get_NON_NEG(fd, gbp, &ndims);
+    err = hdr_get_NON_NEG(fd, gbp, &ndims64);
     if (err != NC_NOERR) {
         if (verbose) printf("Error @ [0x%8.8zx]:\n", err_addr);
         if (verbose) printf("\t%s: Failed to read number of dimensions\n",xloc);
@@ -1647,17 +1656,18 @@ val_get_NC_var(int          fd,
     if (status == NC_NOERR) status = err;
 
     /* cannot be more than NC_MAX_VAR_DIMS */
-    if (ndims > NC_MAX_VAR_DIMS) {
+    if (ndims64 > NC_MAX_VAR_DIMS) {
         if (verbose) printf("Error:\n");
-        if (verbose) printf("\t%s: number of dimensions (%lld) larger than NC_MAX_VAR_DIMS (%d)\n",xloc,ndims,NC_MAX_VAR_DIMS);
+        if (verbose) printf("\t%s: number of dimensions (%lld) larger than NC_MAX_VAR_DIMS (%d)\n",xloc,ndims64,NC_MAX_VAR_DIMS);
         if (name != NULL) free(name);
         DEBUG_RETURN_ERROR(NC_EMAXDIMS)
     }
+    ndims = (int)ndims64;
 
-    if (trace) printf("\t\tnumber of dimensions = %lld\n", ndims);
+    if (trace) printf("\t\tnumber of dimensions = %d\n", ndims);
 
     /* allocate variable object */
-    varp = val_new_NC_var(name, ndims);
+    varp = val_new_NC_var(name, name_len, ndims);
     if (varp == NULL) {
         if (name != NULL) free(name);
         DEBUG_RETURN_ERROR(NC_ENOMEM)
@@ -1868,6 +1878,14 @@ NC_check_vlen(NC_var     *varp,
     return 1;
 }
 
+#define DUMP_DIMS(varp) {                                           \
+    int kk;                                                         \
+    printf("(%lld", ncp->dims.value[varp->dimids[0]]->size);        \
+    for (kk=1; kk<varp->ndims; kk++)                                \
+        printf(", %lld", ncp->dims.value[varp->dimids[kk]]->size);  \
+    printf(")");                                                    \
+}
+
 /*
  * Given a valid ncp, check all variables for their sizes against the maximal
  * allowable sizes. Different CDF formation versions have different maximal
@@ -1907,8 +1925,13 @@ val_NC_check_vlens(NC *ncp)
             if (NC_check_vlen(*vpp, vlen_max) == 0) {
                 /* check this variable's shape product against vlen_max */
                 if (ncp->format >= 5) { /* CDF-5 */
-                    if (verbose) printf("Error:\n");
-                    if (verbose) printf("\tvar %s: variable size greater than max (%lld) allowable by CDF-%d\n",(*vpp)->name,vlen_max,ncp->format);
+                    if (verbose) {
+                        NC_var *varp = (*vpp);
+                        printf("Error:\n");
+                        printf("\tvar %s",varp->name);
+                        DUMP_DIMS(varp)
+                        printf(": variable size greater than max (%lld) allowable by CDF-%d\n",vlen_max,ncp->format);
+                    }
                     DEBUG_RETURN_ERROR(NC_EVARSIZE)
                 }
                 if (!large_fix_vars_count) first_large_fix_var = ii;
@@ -1924,15 +1947,22 @@ val_NC_check_vlens(NC *ncp)
        compute an offset */
     if (large_fix_vars_count > 1) {  /* only one "too-large" variable allowed */
         if (verbose) {
+            NC_var *varp;
             printf("Error:\n");
             printf("\tInput file contains %lld large fixed-size variables\n",large_fix_vars_count);
             printf("\tCDF-%d format allows only one large fixed-size variable\n",ncp->format);
-            printf("\tThe 1st large fixed-size variable is %s\n",ncp->vars.value[first_large_fix_var]->name);
-            printf("\tThe 2nd large fixed-size variable is %s\n",ncp->vars.value[second_large_fix_var]->name);
+            varp = ncp->vars.value[first_large_fix_var];
+            printf("\tThe 1st large fixed-size variable is %s",varp->name);
+            DUMP_DIMS(varp)
+            printf("\n");
+            varp = ncp->vars.value[second_large_fix_var];
+            printf("\tThe 2nd large fixed-size variable is %s\n",varp->name);
+            DUMP_DIMS(varp)
+            printf("\n");
             if (ncp->format == 1)
-                printf("\tSee: https://www.unidata.ucar.edu/software/netcdf/docs/file_structure_and_performance.html#classic_format_limitations\n");
+                printf("\tSee: %s\n", fmt_limit);
             else if (ncp->format == 2)
-                printf("\tSee: http://www.unidata.ucar.edu/software/netcdf/docs/file_structure_and_performance.html#offset_format_limitations\n");
+                printf("\tSee: %s\n", off_limit);
         }
         DEBUG_RETURN_ERROR(NC_EVARSIZE)
     }
@@ -1942,10 +1972,14 @@ val_NC_check_vlens(NC *ncp)
         if (verbose) {
             printf("Error:\n");
             printf("\tCDF-%d format allows only one large fixed-size variable which must be defined last and there is no record variable\n",ncp->format);
+            NC_var *varp = ncp->vars.value[first_large_fix_var];
+            printf("\tThe large fixed-size variable is %s",varp->name);
+            DUMP_DIMS(varp)
+            printf("\n");
             if (ncp->format == 1)
-                printf("\tSee: https://www.unidata.ucar.edu/software/netcdf/docs/file_structure_and_performance.html#classic_format_limitations\n");
+                printf("\tSee: %s\n", fmt_limit);
             else if (ncp->format == 2)
-                printf("\tSee: http://www.unidata.ucar.edu/software/netcdf/docs/file_structure_and_performance.html#offset_format_limitations\n");
+                printf("\tSee: %s\n", off_limit);
         }
         DEBUG_RETURN_ERROR(NC_EVARSIZE)
     }
@@ -1960,9 +1994,9 @@ val_NC_check_vlens(NC *ncp)
             printf("\tInput file contains 1 large fixed-size variables and %lld record variables\n", rec_vars_count);
             printf("\tCDF-%d format allows only one large fixed-size variable which must be defined last and there is no record variable\n",ncp->format);
             if (ncp->format == 1)
-                printf("\tSee: https://www.unidata.ucar.edu/software/netcdf/docs/file_structure_and_performance.html#classic_format_limitations\n");
+                printf("\tSee: %s\n", fmt_limit);
             else if (ncp->format == 2)
-                printf("\tSee: http://www.unidata.ucar.edu/software/netcdf/docs/file_structure_and_performance.html#offset_format_limitations\n");
+                printf("\tSee: %s\n", off_limit);
         }
         DEBUG_RETURN_ERROR(NC_EVARSIZE)
     }
@@ -1976,8 +2010,13 @@ val_NC_check_vlens(NC *ncp)
             if (NC_check_vlen(*vpp, vlen_max) == 0) {
                 /* check this variable's shape product against vlen_max */
                 if (ncp->format >= 5) { /* CDF-5 */
-                    if (verbose) printf("Error:\n");
-                    if (verbose) printf("\tvar %s: variable size greater than max (%lld) allowable by CDF-%d\n",(*vpp)->name,vlen_max,ncp->format);
+                    if (verbose) {
+                        NC_var *varp = (*vpp);
+                        printf("Error:\n");
+                        printf("\tvar %s",varp->name);
+                        DUMP_DIMS(varp)
+                        printf(": variable size greater than max (%lld) allowable by CDF-%d\n",vlen_max,ncp->format);
+                    }
                     DEBUG_RETURN_ERROR(NC_EVARSIZE)
                 }
                 if (!large_rec_vars_count) first_large_rec_var = ii;
@@ -1990,20 +2029,26 @@ val_NC_check_vlens(NC *ncp)
 
     /* For CDF-2, no record variable can require more than 2^32 - 4 bytes of
      * storage for each record's worth of data, unless it is the last record
-     * variable. See
-     * http://www.unidata.ucar.edu/software/netcdf/docs/file_structure_and_performance.html#offset_format_limitations
+     * variable.
      */
     if (large_rec_vars_count > 1) { /* only one "too-large" variable allowed */
         if (verbose) {
+            NC_var *varp;
             printf("Error:\n");
             printf("\tInput file contains %lld large record variables\n",large_rec_vars_count);
-            printf("\tThe 1st large record variable is %s\n",ncp->vars.value[first_large_rec_var]->name);
-            printf("\tThe 2nd large record variable is %s\n",ncp->vars.value[second_large_rec_var]->name);
+            varp = ncp->vars.value[first_large_rec_var];
+            printf("\tThe 1st large record variable is %s",varp->name);
+            DUMP_DIMS(varp)
+            printf("\n");
+            varp = ncp->vars.value[second_large_rec_var];
+            printf("\tThe 2nd large record variable is %s",varp->name);
+            DUMP_DIMS(varp)
+            printf("\n");
             printf("\tCDF-%d format allows only one large record variable\n",ncp->format);
             if (ncp->format == 1)
-                printf("\tSee: https://www.unidata.ucar.edu/software/netcdf/docs/file_structure_and_performance.html#classic_format_limitations\n");
+                printf("\tSee: %s\n", fmt_limit);
             else if (ncp->format == 2)
-                printf("\tSee: http://www.unidata.ucar.edu/software/netcdf/docs/file_structure_and_performance.html#offset_format_limitations\n");
+                printf("\tSee: %s\n", off_limit);
         }
         DEBUG_RETURN_ERROR(NC_EVARSIZE)
     }
@@ -2012,12 +2057,15 @@ val_NC_check_vlens(NC *ncp)
     if (large_rec_vars_count == 1 && last == 0) {
         if (verbose) {
             printf("Error:\n");
-            printf("\tThe 1st large record variable that is not defined last is %s\n",ncp->vars.value[first_large_rec_var]->name);
+            NC_var *varp = ncp->vars.value[first_large_rec_var];
+            printf("\tThe 1st large record variable that is not defined last is %s",varp->name);
+            DUMP_DIMS(varp)
+            printf("\n");
             printf("\tCDF-%d format allows only one large record variable and it must be defined last\n",ncp->format);
             if (ncp->format == 1)
-                printf("\tSee: https://www.unidata.ucar.edu/software/netcdf/docs/file_structure_and_performance.html#classic_format_limitations\n");
+                printf("\tSee: %s\n", fmt_limit);
             else if (ncp->format == 2)
-                printf("\tSee: http://www.unidata.ucar.edu/software/netcdf/docs/file_structure_and_performance.html#offset_format_limitations\n");
+                printf("\tSee: %s\n", off_limit);
         }
         DEBUG_RETURN_ERROR(NC_EVARSIZE)
     }
@@ -2049,7 +2097,7 @@ off_compare(const void *a, const void *b)
 static int
 val_NC_check_voff(NC *ncp)
 {
-    int i, num_fix_vars, nerrs=0, status=NC_NOERR;
+    int i, num_fix_vars, status=NC_NOERR;
 
     if (ncp->vars.ndefined == 0) return NC_NOERR;
 
@@ -2072,7 +2120,6 @@ val_NC_check_voff(NC *ncp)
                 printf("\tvar \"%s\" begin offset (%lld) is less than file header extent (%lld)\n",
                        varp->name, varp->begin, ncp->begin_var);
             }
-            nerrs++;
             DEBUG_ASSIGN_ERROR(status, NC_ENOTNC)
         }
         if (IS_RECVAR(varp)) continue;
@@ -2102,7 +2149,6 @@ val_NC_check_voff(NC *ncp)
                 printf("\tvar \"%s\" begin offset (%lld) overlaps var %s (begin=%lld, length=%lld)\n",
                        var_cur->name, var_cur->begin, var_prv->name, var_prv->begin, var_prv->len);
             }
-            nerrs++;
             DEBUG_ASSIGN_ERROR(status, NC_ENOTNC)
         }
         var_end = var_off_len[i].off + var_off_len[i].len;
@@ -2112,7 +2158,6 @@ val_NC_check_voff(NC *ncp)
     if (ncp->begin_rec < max_var_end) {
         if (verbose) printf("Error:\n");
         if (verbose) printf("\tRecord variable section begin offset (%lld) is less than fixed-size variable section end offset (%lld)\n", ncp->begin_rec, max_var_end);
-        nerrs++;
         DEBUG_ASSIGN_ERROR(status, NC_ENOTNC)
     }
     free(var_off_len);
@@ -2150,7 +2195,6 @@ check_rec_var:
                 printf("\tvar \"%s\" begin offset (%lld) overlaps var %s (begin=%lld, length=%lld)\n",
                        var_cur->name, var_cur->begin, var_prv->name, var_prv->begin, var_prv->len);
             }
-            nerrs++;
             DEBUG_ASSIGN_ERROR(status, NC_ENOTNC)
         }
     }
@@ -2174,7 +2218,6 @@ check_rec_var:
                 else
                     printf("\tvar \"%s\" begin offset (%lld) is less than previous variable \"%s\" end offset (%lld)\n", varp->name, varp->begin, ncp->vars.value[prev]->name, prev_off);
             }
-            nerrs++;
             DEBUG_ASSIGN_ERROR(status, NC_ENOTNC)
         }
         prev_off = varp->begin + varp->len;
@@ -2184,7 +2227,6 @@ check_rec_var:
     if (ncp->begin_rec < prev_off) {
         if (verbose) printf("Error:\n");
         if (verbose) printf("\tRecord variable section begin offset (%lld) is less than fixed-size variable section end offset (%lld)\n", ncp->begin_rec, prev_off);
-        nerrs++;
         DEBUG_ASSIGN_ERROR(status, NC_ENOTNC)
     }
 
@@ -2206,7 +2248,6 @@ check_rec_var:
                 else
                     printf("Variable \"%s\" begin offset (%lld) is less than previous variable \"%s\" end offset (%lld)\n", varp->name, varp->begin, ncp->vars.value[prev]->name, prev_off);
             }
-            nerrs++;
             DEBUG_ASSIGN_ERROR(status, NC_ENOTNC)
         }
         prev_off = varp->begin + varp->len;
@@ -2405,6 +2446,37 @@ fn_exit:
 
 /* End Of get NC */
 
+/* File system types recognized by ROMIO in MPICH 4.0.0 */
+static const char* fstypes[] = {"ufs", "nfs", "xfs", "pvfs2", "gpfs", "panfs", "lustre", "daos", "testfs", "ime", "quobyte", NULL};
+
+/* Return a pointer to filename by removing the file system type prefix name if
+ * there is any.  For example, when filename = "lustre:/home/foo/testfile.nc",
+ * remove "lustre:" to return a pointer to "/home/foo/testfile.nc", so the name
+ * can be used in POSIX open() calls.
+ */
+static char*
+remove_file_system_type_prefix(const char *filename)
+{
+    char *ret_filename = (char*)filename;
+
+    if (filename == NULL) return NULL;
+
+    if (strchr(filename, ':') != NULL) { /* there is a prefix end with ':' */
+        /* check if prefix is one of recognized file system types */
+        int i=0;
+        while (fstypes[i] != NULL) {
+            size_t prefix_len = strlen(fstypes[i]);
+            if (!strncmp(filename, fstypes[i], prefix_len)) { /* found */
+                ret_filename += prefix_len + 1;
+                break;
+            }
+            i++;
+        }
+    }
+
+    return ret_filename;
+}
+
 static int
 check_signature(char *filename)
 {
@@ -2482,7 +2554,7 @@ usage(char *argv0)
     "       [-x] Repair in-place the null-byte padding in file header.\n"
     "       [-q] Quiet mode (exit 1 when fail, 0 success)\n"
     "       file: Input netCDF file name\n"
-    "*PnetCDF library version 1.12.3 of February 21, 2022\n";
+    "*PnetCDF library version PNETCDF_RELEASE_VERSION of PNETCDF_RELEASE_DATE\n";
     fprintf(stderr, help, argv0);
 }
 
@@ -2522,9 +2594,7 @@ int main(int argc, char **argv)
      * For example, when filename = "lustre:/home/foo/testfile.nc", remove
      * "lustre:" to make path = "/home/foo/testfile.nc" in open() below
      */
-    path = strchr(filename, ':');
-    if (path == NULL) path = filename; /* no prefix */
-    else              path++;
+    path = remove_file_system_type_prefix(filename);
 
     /* check file signature */
     fmt = check_signature(path);
@@ -2561,7 +2631,7 @@ int main(int argc, char **argv)
     /* class file format: CDF-1, 2 or 5 */
     fmt = ncp->format;
 
-    /* check data size */
+    /* check file size */
     if (-1 == fstat(fd, &ncfilestat)) {
         if (verbose) printf("Error at line %d fstat (%s)\n",__LINE__,strerror(errno));
         status = NC_EFILE;
@@ -2571,10 +2641,10 @@ int main(int argc, char **argv)
         long long expect_fsize;
         expect_fsize = ncp->begin_rec + ncp->recsize * ncp->numrecs;
         if (expect_fsize < ncfilestat.st_size) {
-            if (verbose) printf("Error: file size (%lld) is larger than expected (%lld)!\n",(long long)ncfilestat.st_size, expect_fsize);
-            if (verbose) printf("\tbegin_rec=%lld recsize=%lld numrecs=%lld ncfilestat.st_size=%lld\n",ncp->begin_rec, ncp->recsize, ncp->numrecs, (long long) ncfilestat.st_size);
-            status = NC_EFILE;
-            goto prog_exit;
+            if (verbose) {
+                printf("Warning: file size (%lld) is larger than expected (%lld)!\n",(long long)ncfilestat.st_size, expect_fsize);
+                printf("\tbegin_rec=%lld recsize=%lld numrecs=%lld ncfilestat.st_size=%lld\n",ncp->begin_rec, ncp->recsize, ncp->numrecs, (long long) ncfilestat.st_size);
+            }
         }
         else if (expect_fsize > ncfilestat.st_size) {
             /* if file header are valid and the only error is the file size
@@ -2587,21 +2657,22 @@ int main(int argc, char **argv)
         }
     }
     else { /* either there is no record variable or no record is written */
-	/* Assuming variables' begins do not follow their define order, find
+        /* Assuming variables' begins do not follow their define order, find
          * max end offset among all fixed-size varaibles.
          */
-        long long expect_fsize = ncp->xsz;
-	for (i=0; i<ncp->vars.ndefined; i++) {
+        long long expect_fsize = MAX(ncp->begin_var, ncp->begin_rec);
+        expect_fsize = MAX(expect_fsize, ncp->xsz);
+        for (i=0; i<ncp->vars.ndefined; i++) {
             long long var_end;
             if (IS_RECVAR(ncp->vars.value[i])) continue;
             var_end = ncp->vars.value[i]->begin + ncp->vars.value[i]->len;
             expect_fsize = MAX(expect_fsize, var_end);
         }
         if (expect_fsize < ncfilestat.st_size) {
-            if (verbose) printf("Error:\n");
-            if (verbose) printf("\tfile size (%lld) is larger than expected (%lld)!\n",(long long)ncfilestat.st_size, expect_fsize);
-            status = NC_EFILE;
-            goto prog_exit;
+            if (verbose) {
+                printf("Warning:\n");
+                printf("\tfile size (%lld) is larger than expected (%lld)!\n",(long long)ncfilestat.st_size, expect_fsize);
+            }
         }
         else if (expect_fsize > ncfilestat.st_size) {
             /* if file header are valid and the only error is the file size
