@@ -24,6 +24,24 @@
     } \
 }
 
+#define CHECK_ERROUT { \
+    if (err != NC_NOERR) { \
+        nerrs++; \
+        printf("Error at line %d in %s: (%s)\n", \
+        __LINE__,__FILE__,ncmpi_strerrno(err)); \
+        goto err_out; \
+    } \
+}
+
+#define CHECK_FATAL_ERR { \
+    if (err != NC_NOERR) { \
+        nerrs++; \
+        printf("Error at line %d in %s: (%s)\n", \
+        __LINE__,__FILE__,ncmpi_strerrno(err)); \
+        MPI_Abort(MPI_COMM_WORLD, -1); \
+    } \
+}
+
 #define EXP_ERR(exp) { \
     if (err != exp) { \
         nerrs++; \
@@ -68,4 +86,6 @@ extern char *strdup(const char *s);
 extern int strcasecmp(const char *s1, const char *s2);
 #endif
 
+char* remove_file_system_type_prefix(const char *filename);
+
 #endif
