@@ -35,7 +35,6 @@ int main(int argc, char** argv)
     MPI_Offset start[2], count[2];
     MPI_Info info;
     size_t i;
-    int bb_enabled=0;
 
     MPI_Init(&argc, &argv);
     MPI_Comm_rank(MPI_COMM_WORLD, &rank);
@@ -69,11 +68,8 @@ int main(int argc, char** argv)
     MPI_Info_set(info, "romio_cb_write", "enable");
     MPI_Info_set(info, "romio_ds_read", "disable"); /* run slow without it */
 
-#if defined(ENABLE_LARGE_SINGLE_REQ) || defined(ENABLE_BURST_BUFFER)
-#else
     /* silence iternal debug messages */
     setenv("PNETCDF_SAFE_MODE", "0", 1);
-#endif
 
 #ifdef ENABLE_NETCDF4
     /* Test for NetCDF 4 first as ncvalidator checks only read classic files */
@@ -153,18 +149,6 @@ int main(int argc, char** argv)
     CHECK_ERR
     MPI_Info_free(&info);
 
-    {
-        int flag;
-        char hint[MPI_MAX_INFO_VAL];
-        MPI_Info infoused;
-
-        ncmpi_inq_file_info(ncid, &infoused);
-        MPI_Info_get(infoused, "nc_burst_buf", MPI_MAX_INFO_VAL - 1, hint, &flag);
-        if (flag && strcasecmp(hint, "enable") == 0)
-            bb_enabled = 1;
-        MPI_Info_free(&infoused);
-    }
-
     /* define dimensions */
     err = ncmpi_def_dim(ncid, "NPROCS", nprocs, &dimid[0]);
     CHECK_ERR
@@ -181,16 +165,8 @@ int main(int argc, char** argv)
     CHECK_ERR
 
     /* now we are in data mode */
-#if defined(ENABLE_LARGE_SINGLE_REQ) || defined(ENABLE_BURST_BUFFER)
-#ifndef ENABLE_LARGE_SINGLE_REQ
-    if (bb_enabled) {
-#endif
     for (i=0; i<20; i++) buf[ONE_G-10+i] = 'a'+i;
     for (i=0; i<20; i++) buf[TWO_G-10+i] = 'A'+i;
-#ifndef ENABLE_LARGE_SINGLE_REQ
-    }
-#endif
-#endif
 
     start[0] = rank;
     count[0] = 1;
@@ -215,10 +191,6 @@ int main(int argc, char** argv)
     CHECK_ERR
 
     err = ncmpi_wait_all(ncid, 3, req, st);
-#if defined(ENABLE_LARGE_SINGLE_REQ) || defined(ENABLE_BURST_BUFFER)
-#ifndef ENABLE_LARGE_SINGLE_REQ
-    if (bb_enabled) {
-#endif
     CHECK_ERR
 
     /* read back to check contents */
@@ -249,14 +221,6 @@ int main(int argc, char** argv)
 
     /* test the same pattern but for iget */
     for (i=0; i<TWO_G+1024; i++) buf[i] = 0;
-#ifndef ENABLE_LARGE_SINGLE_REQ
-    }
-    else
-        EXP_ERR(NC_EMAX_REQ)
-#endif
-#else
-    EXP_ERR(NC_EMAX_REQ)
-#endif
 
     start[1] = 0;
     count[1] = 10;
@@ -274,9 +238,6 @@ int main(int argc, char** argv)
     CHECK_ERR
 
     err = ncmpi_wait_all(ncid, 3, req, st);
-#ifndef ENABLE_LARGE_SINGLE_REQ
-    EXP_ERR(NC_EMAX_REQ)
-#else
     CHECK_ERR
 
     for (i=0; i<20; i++) {
@@ -294,7 +255,6 @@ int main(int argc, char** argv)
             nerrs++;
         }
     }
-#endif
 
     err = ncmpi_close(ncid); CHECK_ERR
 
